@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2005-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 2005-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -37,7 +37,7 @@ public:
     Window();
 
 private:
-    QSlider *createSlider(const char *changedSignal, const char *setterSlot);
+    QSlider *createSlider();
 
     GLWidget *glWidget;
     QSlider *xSlider;
