@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2004-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 2004-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the example classes of the Qt Toolkit.
 **
@@ -30,5 +30,5 @@ int main(int argc, char *argv[])
     QApplication app(argc, argv);
     Dialog dialog;
     dialog.show();
-    return dialog.exec();
+    return app.exec();
 }
