@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -223,7 +223,8 @@ QByteArray QUtf16Codec::convertFromUnicode(const QChar *uc, int len, ConverterSt
     int length =  2*len;
     if (!state || (!(state->flags & IgnoreHeader))) {
         length += 2;
-    } else if (e == Detect) {
+    }
+    if (e == Detect) {
         endian = (QSysInfo::ByteOrder == QSysInfo::BigEndian) ? BE : LE;
     }
 
@@ -380,5 +381,4 @@ QList<QByteArray> QUtf16LECodec::aliases() const
     return list;
 }
 
-
 #endif //QT_NO_TEXTCODEC
