@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -25,6 +25,7 @@
 #include "qfileinfo.h"
 #include "qdatetime.h"
 #include "qabstractfileengine.h"
+#include "qfsfileengine_p.h"
 #include "qglobal.h"
 #include "qatomic.h"
 #include "qhash.h"
@@ -38,11 +39,18 @@ public:
 
     void initFileEngine(const QString &);
 
+    enum Access {
+        ReadAccess,
+        WriteAccess,
+        ExecuteAccess
+    };
+    bool hasAccess(Access access) const;
+
     uint getFileFlags(QAbstractFileEngine::FileFlags) const;
     QDateTime &getFileTime(QAbstractFileEngine::FileTime) const;
     QString getFileName(QAbstractFileEngine::FileName) const;
 
-    enum { CachedPerms=0x01, CachedTypes=0x02, CachedFlags=0x04,
+    enum { CachedFileFlags=0x01, CachedLinkTypeFlag=0x02,
            CachedMTime=0x10, CachedCTime=0x20, CachedATime=0x40,
            CachedSize =0x08 };
     struct Data {
@@ -50,7 +58,8 @@ public:
             : ref(1), fileEngine(0), cache_enabled(1)
         { clear(); }
         inline Data(const Data &copy)
-            : ref(1), fileEngine(0), fileName(copy.fileName)
+            : ref(1), fileEngine(QAbstractFileEngine::create(copy.fileName)),
+              fileName(copy.fileName), cache_enabled(copy.cache_enabled)
         { clear(); }
         inline ~Data() { delete fileEngine; }
         inline void clear() {
@@ -64,13 +73,15 @@ public:
         mutable QString fileName;
         mutable QHash<int, QString> fileNames;
 
+        mutable uint cachedFlags : 31;
         mutable uint cache_enabled : 1;
+        mutable uint fileFlags;
         mutable qint64 fileSize;
         mutable QDateTime fileTimes[3];
-        mutable uint fileFlags;
-        mutable uchar cachedFlags;
-        inline bool getCachedFlag(uchar c) const { return cache_enabled ? (cachedFlags&c) : 0; }
-        inline void setCachedFlag(uchar c) { if(cache_enabled) cachedFlags |= c; }
+        inline bool getCachedFlag(uint c) const
+        { return cache_enabled ? (cachedFlags & c) : 0; }
+        inline void setCachedFlag(uint c)
+        { if (cache_enabled) cachedFlags |= c; }
     } *data;
     inline void reset() {
         detach();
@@ -108,6 +119,53 @@ QFileInfoPrivate::initFileEngine(const QString &file)
     data->fileName = file;
 }
 
+bool QFileInfoPrivate::hasAccess(Access access) const
+{
+    if (!(data->fileEngine->fileFlags() & QAbstractFileEngine::LocalDiskFlag)) {
+        switch (access) {
+        case ReadAccess:
+            return getFileFlags(QAbstractFileEngine::ReadUserPerm);
+        case WriteAccess:
+            return getFileFlags(QAbstractFileEngine::WriteUserPerm);
+        case ExecuteAccess:
+            return getFileFlags(QAbstractFileEngine::ExeUserPerm);
+        default:
+            return false;
+        }
+    }
+
+    int mode = 0;
+    switch (access) {
+    case ReadAccess:
+        mode = R_OK;
+        break;
+    case WriteAccess:
+        mode = W_OK;
+        break;
+    case ExecuteAccess:
+        mode = X_OK;
+        break;
+    };
+#ifdef Q_OS_UNIX
+    return QT_ACCESS(QFile::encodeName(data->fileName).data(), mode) == 0;
+#endif
+#ifdef Q_OS_WIN
+    if ((access == ReadAccess && !getFileFlags(QAbstractFileEngine::ReadUserPerm))
+        || (access == WriteAccess && !getFileFlags(QAbstractFileEngine::WriteUserPerm))) {
+        return false;
+    }
+    if (access == ExecuteAccess)
+        return getFileFlags(QAbstractFileEngine::ExeUserPerm);
+
+    QT_WA( {
+        return ::_waccess((TCHAR *)QFSFileEnginePrivate::longFileName(data->fileName).utf16(), mode) == 0;
+    } , {
+        return QT_ACCESS(QFSFileEnginePrivate::win95Name(data->fileName), mode) == 0;
+    } );
+#endif
+    return false;
+}
+
 void QFileInfoPrivate::detach()
 { qAtomicDetach(data); }
 
@@ -125,22 +183,36 @@ QFileInfoPrivate::getFileName(QAbstractFileEngine::FileName name) const
 uint
 QFileInfoPrivate::getFileFlags(QAbstractFileEngine::FileFlags request) const
 {
-    QAbstractFileEngine::FileFlags flags = 0;
-    if((request & QAbstractFileEngine::TypesMask) && !data->getCachedFlag(CachedTypes)) {
-        data->setCachedFlag(CachedTypes);
-        flags |= QAbstractFileEngine::TypesMask;
+    // we split the testing for LinkType and the rest because, in order to
+    // determine if a file is a symlink or not, we have to lstat(). If we're not
+    // interested in that information, we might as well avoid one extra syscall.
+
+    QAbstractFileEngine::FileFlags flags;
+    if (!data->getCachedFlag(CachedFileFlags)) {
+        QAbstractFileEngine::FileFlags req = QAbstractFileEngine::FileInfoAll;
+        req &= (~QAbstractFileEngine::LinkType);
+
+        flags = data->fileEngine->fileFlags(req);
+        data->setCachedFlag(CachedFileFlags);
+        data->fileFlags |= uint(flags);
+    } else {
+        flags = QAbstractFileEngine::FileFlags(data->fileFlags & request);
     }
-    if((request & QAbstractFileEngine::PermsMask) && !data->getCachedFlag(CachedPerms)) {
-        data->setCachedFlag(CachedPerms);
-        flags |= QAbstractFileEngine::PermsMask;
+
+    if (request & QAbstractFileEngine::LinkType) {
+        if (!data->getCachedFlag(CachedLinkTypeFlag)) {
+            QAbstractFileEngine::FileFlags linkflag;
+            linkflag = data->fileEngine->fileFlags(QAbstractFileEngine::LinkType);
+
+            data->setCachedFlag(CachedLinkTypeFlag);
+            data->fileFlags |= uint(linkflag);
+            flags |= linkflag;
+        }
     }
-    if((request & QAbstractFileEngine::FlagsMask) && !data->getCachedFlag(CachedFlags)) {
-        data->setCachedFlag(CachedFlags);
-        flags |= QAbstractFileEngine::FlagsMask;
-    }
-    if(flags)
-        data->fileFlags |= (data->fileEngine->fileFlags(flags) & flags);
-    return data->fileFlags & request;
+    // no else branch
+    // if we had it cached, it was caught in the previous else branch
+
+    return flags & request;
 }
 
 QDateTime
@@ -175,6 +247,7 @@ QDateTime
     \brief The QFileInfo class provides system-independent file information.
 
     \ingroup io
+    \ingroup shared
 
     QFileInfo provides information about a file's name and position
     (path) in the file system, its access rights and whether it is a
@@ -198,6 +271,16 @@ QDateTime
     later with setFile(). Use exists() to see if the file exists and
     size() to get its size.
 
+    Some of QFileInfo's functions query the file system, but for
+    performance reasons, some functions only operate on the
+    file name itself. For example: To return the absolute path of
+    a relative file name, absolutePath() has to query the file system.
+    The path() function, however, can work on the file name directly,
+    and so it is faster. By convention, QFileInfo interprets any path that
+    ends with a slash '/' as a directory (e.g., "C:/WINDOWS/"), and
+    those without a trailing slash (e.g., "C:/WINDOWS/hosts.txt")
+    are treated as files.
+    
     To speed up performance, QFileInfo caches information about the
     file. Because files can be changed by other users or programs, or
     even by other parts of the same program, there is a function that
@@ -206,9 +289,53 @@ QDateTime
     every time you request information from it call setCaching(false).
 
     The file's type is obtained with isFile(), isDir() and
-    isSymLink(). The readLink() function provides the name of the file
+    isSymLink(). The symLinkTarget() function provides the name of the file
     the symlink points to.
 
+    On Unix (including Mac OS X), the symlink has the same size() has
+    the file it points to, because Unix handles symlinks
+    transparently; similarly, opening a symlink using QFile
+    effectively opens the link's target. For example:
+
+    \code
+        #ifdef Q_OS_UNIX
+
+        QFileInfo info1("/home/bob/bin/untabify");
+        info1.isSymLink();          // returns true
+        info1.absoluteFilePath();   // returns "/home/bob/bin/untabify"
+        info1.size();               // returns 56201
+        info1.symLinkTarget();      // returns "/opt/pretty++/bin/untabify"
+
+        QFileInfo info2(info1.symLinkTarget());
+        info1.isSymLink();          // returns false
+        info1.absoluteFilePath();   // returns "/opt/pretty++/bin/untabify"
+        info1.size();               // returns 56201
+
+        #endif
+    \endcode
+
+    On Windows, symlinks (shortcuts) are \c .lnk files. The reported
+    size() is that of the symlink (not the link's target), and
+    opening a symlink using QFile opens the \c .lnk file. For
+    example:
+
+    \code
+        #ifdef Q_OS_WIN
+
+        QFileInfo info1("C:\\Documents and Settings\\Bob\\untabify.lnk");
+        info1.isSymLink();          // returns true
+        info1.absoluteFilePath();   // returns "C:/Documents and Settings/Bob/untabify.lnk"
+        info1.size();               // returns 743
+        info1.symLinkTarget();      // returns "C:/Pretty++/untabify"
+
+        QFileInfo info2(info1.symLinkTarget());
+        info1.isSymLink();          // returns false
+        info1.absoluteFilePath();   // returns "C:/Pretty++/untabify"
+        info1.size();               // returns 63942
+
+        #endif
+    \endcode
+
     Elements of the file's name can be extracted with path() and
     fileName(). The fileName()'s parts can be extracted with
     baseName() and extension().
@@ -224,7 +351,11 @@ QDateTime
 */
 
 /*!
-    Constructs a new empty QFileInfo.
+    Constructs an empty QFileInfo object.
+
+    Note that an empty QFileInfo object contain no file reference.
+
+    \sa setFile()
 */
 
 QFileInfo::QFileInfo() : d_ptr(new QFileInfoPrivate())
@@ -296,26 +427,27 @@ QFileInfo::~QFileInfo()
 /*!
     \fn bool QFileInfo::operator!=(const QFileInfo &fileinfo)
 
-    Returns true if the QFileInfo refers to a different file to the one
-    specified by \a fileinfo; otherwise returns false.
+    Returns true if this QFileInfo object refers to a different file
+    than the one specified by \a fileinfo; otherwise returns false.
 
     \sa operator==()
 */
 
 /*!
-    Returns true if the QFileInfo refers to a file in the same location as
-    the other \a fileinfo; otherwise returns false.
+    \overload
+    \fn bool QFileInfo::operator!=(const QFileInfo &fileinfo) const
+*/
 
-    \warning This will not compare two different symbolic links
-    pointing to the same file.
-
-    \sa operator!=()
+/*!
+    \overload
 */
 
 bool
 QFileInfo::operator==(const QFileInfo &fileinfo) const
 {
     Q_D(const QFileInfo);
+    // ### Qt 5: understand long and short file names on Windows
+    // ### (GetFullPathName()).
     if(fileinfo.d_func()->data == d->data)
         return true;
     if(!d->data->fileEngine || !fileinfo.d_func()->data->fileEngine)
@@ -338,6 +470,22 @@ QFileInfo::operator==(const QFileInfo &fileinfo) const
     }
     return false;
 }
+
+/*!
+    Returns true if this QFileInfo object refers to a file in the same
+    location as \a fileinfo; otherwise returns false.
+
+    Note that the result of comparing two empty QFileInfo objects,
+    containing no file references, is undefined.
+
+    \warning This will not compare two different symbolic links
+    pointing to the same file.
+
+    \warning Long and short file names that refer to the same file on Windows
+    are treated as if they referred to different files.
+
+    \sa operator!=()
+*/
 bool QFileInfo::operator==(const QFileInfo &fileinfo)
 {
     return const_cast<const QFileInfo *>(this)->operator==(fileinfo);
@@ -559,9 +707,9 @@ QFileInfo::isRelative() const
 
 
 /*!
-    Converts the file's path to an absolute path.
-
-    If it is already absolute, nothing is done.
+    Converts the file's path to an absolute path if it is not already in that form.
+    Returns true to indicate that the path was converted; otherwise returns false
+    to indicate that the path was already absolute.
 
     \sa filePath(), isRelative()
 */
@@ -651,6 +799,11 @@ QFileInfo::fileName() const
         QString base = fi.baseName();  // base = "archive"
     \endcode
 
+
+    The base name of a file is computed equally on all platforms, independent
+    of file naming conventions (e.g., ".bashrc" on Unix has an empty base
+    name, and the suffix is "bashrc").
+
     \sa fileName(), suffix(), completeSuffix(), completeBaseName()
 */
 
@@ -684,7 +837,9 @@ QFileInfo::completeBaseName() const
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
         return QLatin1String("");
-    return d->getFileName(QAbstractFileEngine::BaseName).section(QLatin1Char('.'), 0, -2);
+    QString name = d->getFileName(QAbstractFileEngine::BaseName);
+    int index = name.lastIndexOf(QLatin1Char('.'));
+    return (index == -1) ? name : name.left(index);
 }
 
 /*!
@@ -727,6 +882,10 @@ QFileInfo::completeSuffix() const
         QString ext = fi.suffix();  // ext = "gz"
     \endcode
 
+    The suffix of a file is computed equally on all platforms, independent of
+    file naming conventions (e.g., ".bashrc" on Unix has an empty base name,
+    and the suffix is "bashrc").
+
     \sa fileName(), completeSuffix(), baseName(), completeBaseName()
 */
 
@@ -745,7 +904,10 @@ QFileInfo::suffix() const
 
 
 /*!
-    Returns the file's path as a QDir object.
+    Returns the path of the object's parent directory as a QDir object.
+
+    \bold{Note:} The QDir returned always corresponds to the object's parent
+    directory, even if the QFileInfo represents a directory.
 
     \sa dirPath(), filePath(), fileName(), isRelative(), absoluteDir()
 */
@@ -793,7 +955,7 @@ QFileInfo::isReadable() const
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
         return false;
-    return d->getFileFlags(QAbstractFileEngine::ReadUserPerm);
+    return d->hasAccess(QFileInfoPrivate::ReadAccess);
 }
 
 /*!
@@ -809,7 +971,7 @@ QFileInfo::isWritable() const
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
         return false;
-    return d->getFileFlags(QAbstractFileEngine::WriteUserPerm);
+    return d->hasAccess(QFileInfoPrivate::WriteAccess);
 }
 
 /*!
@@ -824,7 +986,7 @@ QFileInfo::isExecutable() const
     Q_D(const QFileInfo);
     if(!d->data->fileEngine)
         return false;
-    return d->getFileFlags(QAbstractFileEngine::ExeUserPerm);
+    return d->hasAccess(QFileInfoPrivate::ExecuteAccess);
 }
 
 /*!
@@ -876,7 +1038,19 @@ QFileInfo::isDir() const
     Returns true if this object points to a symbolic link (or to a
     shortcut on Windows); otherwise returns false.
 
-    \sa isFile(), isDir(), readLink()
+    On Unix (including Mac OS X), opening a symlink effectively opens
+    the \l{symLinkTarget()}{link's target}. On Windows, it opens the \c
+    .lnk file itself.
+
+    Example:
+
+    \code
+        QFileInfo info(fileName);
+        if (info.isSymLink())
+            fileName = info.symLinkTarget();
+    \endcode
+
+    \sa isFile(), isDir(), symLinkTarget()
 */
 
 bool
@@ -904,8 +1078,12 @@ QFileInfo::isRoot() const
 }
 
 /*!
-    Returns the name a symlink (or shortcut on Windows) points to, or
-    a an empty string if the object isn't a symbolic link.
+    \fn QString QFileInfo::symLinkTarget() const
+    \since 4.2
+
+    Returns the absolute path to the file or directory a symlink (or shortcut
+    on Windows) points to, or a an empty string if the object isn't a symbolic
+    link.
 
     This name may not represent an existing file; it is only a string.
     QFileInfo::exists() returns true if the symlink points to an
@@ -914,6 +1092,11 @@ QFileInfo::isRoot() const
     \sa exists(), isSymLink(), isDir(), isFile()
 */
 
+/*!
+    \obsolete
+
+    Use symLinkTarget() instead.
+*/
 QString
 QFileInfo::readLink() const
 {
@@ -1044,8 +1227,10 @@ QFileInfo::permissions() const
 
 
 /*!
-    Returns the file size in bytes, or 0 if the file does not exist or
-    if the size is 0 or if the size cannot be fetched.
+    Returns the file size in bytes. If the file does not exist or cannot be
+    fetched, 0 is returned.
+
+    \sa exists()
 */
 
 qint64
@@ -1211,3 +1396,10 @@ QFileInfo::setCaching(bool enable)
 
     Use permission() instead.
 */
+
+/*!
+    \typedef QFileInfoList
+    \relates QFileInfo
+
+    Synonym for QList<QFileInfo>.
+*/
