@@ -1,10 +1,10 @@
 /****************************************************************************
- **
- ** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
- **
- ** This file is part of the QtCore module of the Qt Toolkit.
- **
- ** This file may be used under the terms of the GNU General Public
+**
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
+**
+** This file is part of the QtCore module of the Qt Toolkit.
+**
+** This file may be used under the terms of the GNU General Public
 ** License version 2.0 as published by the Free Software Foundation
 ** and appearing in the file LICENSE.GPL included in the packaging of
 ** this file.  Please review the following information to ensure GNU
@@ -15,11 +15,11 @@
 ** review the following information:
 ** http://www.trolltech.com/products/qt/licensing.html or contact the
 ** sales department at sales@trolltech.com.
- **
- ** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
- ** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
- **
- ****************************************************************************/
+**
+** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
+** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+**
+****************************************************************************/
 
 #ifndef QDATETIME_P_H
 #define QDATETIME_P_H
@@ -41,13 +41,15 @@
 #include "QtCore/qstringlist.h"
 #include "QtCore/qlist.h"
 
-#define QTIME_MIN QTime(0, 0, 0, 0)
-#define QTIME_MAX QTime(23, 59, 59, 999)
-#define QDATE_MIN QDate(1752, 9, 14)
-#define QDATE_MAX QDate(7999, 12, 31)
-#define QDATETIME_MIN QDateTime(QDATE_MIN, QTIME_MIN)
-#define QDATETIME_MAX QDateTime(QDATE_MAX, QTIME_MAX)
-#define QDATE_INITIAL QDate(2000, 1, 1)
+#define QDATETIMEEDIT_TIME_MIN QTime(0, 0, 0, 0)
+#define QDATETIMEEDIT_TIME_MAX QTime(23, 59, 59, 999)
+#define QDATETIMEEDIT_DATE_MIN QDate(100, 1, 1)
+#define QDATETIMEEDIT_COMPAT_DATE_MIN QDate(1752, 9, 14)
+#define QDATETIMEEDIT_DATE_MAX QDate(7999, 12, 31)
+#define QDATETIMEEDIT_DATETIME_MIN QDateTime(QDATETIMEEDIT_DATE_MIN, QDATETIMEEDIT_TIME_MIN)
+#define QDATETIMEEDIT_COMPAT_DATETIME_MIN QDateTime(QDATETIMEEDIT_COMPAT_DATE_MIN, QDATETIMEEDIT_TIME_MIN)
+#define QDATETIMEEDIT_DATETIME_MAX QDateTime(QDATETIMEEDIT_DATE_MAX, QDATETIMEEDIT_TIME_MAX)
+#define QDATETIMEEDIT_DATE_INITIAL QDate(2000, 1, 1)
 
 
 class QDateTimePrivate
@@ -124,11 +126,11 @@ public:
     }; // duplicated from qdatetimeedit.h
     Q_DECLARE_FLAGS(Sections, Section)
 
-    struct SectionNode {
-        Section type;
-        mutable int pos;
-        int count;
-    };
+        struct SectionNode {
+            Section type;
+            mutable int pos;
+            int count;
+        };
 
     enum State { // duplicated from QValidator
         Invalid,
@@ -186,9 +188,8 @@ public:
 #endif
     int findAmPm(QString &str1, int index, int *used = 0) const;
     int maxChange(int s) const;
-    int potentialValue(const QString &str, int min, int max, int index, const QVariant &currentValue) const;
-    int potentialValueHelper(const QString &str, int min, int max, int size) const;
-    int multiplier(int s) const;
+    int potentialValue(const QString &str, int min, int max, int index, const QVariant &currentValue, int insert) const;
+    int potentialValueHelper(const QString &str, int min, int max, int size, int insert) const;
 
     QString sectionName(int s) const;
     QString stateName(int s) const;
@@ -196,11 +197,19 @@ public:
     QString sectionFormat(int index) const;
     QString sectionFormat(Section s, int count) const;
 
-    bool isFixedNumericSection(int index) const;
+    enum FieldInfoFlag {
+        Numeric = 0x01,
+        FixedWidth = 0x02,
+        AllowPartial = 0x04
+    };
+    Q_DECLARE_FLAGS(FieldInfo, FieldInfoFlag)
+
+    FieldInfo fieldInfo(int index) const;
 
 #ifndef QT_NO_DATESTRING
     virtual QVariant getMinimum() const;
     virtual QVariant getMaximum() const;
+    virtual int cursorPosition() const { return -1; }
 #endif
     virtual QString displayText() const { return text; }
     virtual QString getAmPmText(AmPm ap, Case cs) const;
@@ -225,6 +234,8 @@ public:
 Q_CORE_EXPORT bool operator==(const QDateTimeParser::SectionNode &s1, const QDateTimeParser::SectionNode &s2);
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QDateTimeParser::Sections)
+Q_DECLARE_OPERATORS_FOR_FLAGS(QDateTimeParser::FieldInfo)
+
 
 #endif // QT_BOOTSTRAPPED
 
