@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
@@ -26,12 +26,12 @@
 /*! \typedef QStringListIterator
     \relates QStringList
 
-    The QStringListIterator typedef provides a Java-style const
+    The QStringListIterator type definition provides a Java-style const
     iterator for QStringList.
 
     QStringList provides both \l{Java-style iterators} and
     \l{STL-style iterators}. The Java-style const iterator is simply
-    a typedef for QListIterator<QString>.
+    a type definition for QListIterator<QString>.
 
     \sa QMutableStringListIterator, QStringList::const_iterator
 */
@@ -39,12 +39,12 @@
 /*! \typedef QMutableStringListIterator
     \relates QStringList
 
-    The QStringListIterator typedef provides a Java-style non-const
-    iterator for QStringList.
+    The QStringListIterator type definition provides a Java-style
+    non-const iterator for QStringList.
 
     QStringList provides both \l{Java-style iterators} and
     \l{STL-style iterators}. The Java-style non-const iterator is
-    simply a typedef for QMutableListIterator<QString>.
+    simply a type definition for QMutableListIterator<QString>.
 
     \sa QStringListIterator, QStringList::iterator
 */
@@ -59,97 +59,107 @@
     \mainclass
     \reentrant
 
-    QStringList inherits from QList\<QString\>. All of QList's
-    functionality also applies to QStringList. For example, you can
-    use isEmpty() to test whether the list is empty, and you can call
-    functions like append(), prepend(), insert(), replace(), and
-    remove() to modify a QStringList. In addition, QStringList
-    provides a few convenience functions that make handling lists of
-    strings easier.
+    QStringList inherits from QList<QString>. Like QList, QStringList
+    is \l{implicitly shared}. It provides fast index-based access as
+    well as fast insertions and removals. Passing string lists as
+    value parameters is both fast and safe.
 
-    Like QList, QStringList is \l{implicitly shared}. QStringList
-    provides fast index-based access as well as fast insertions and
-    removals. Passing string lists as value parameters is both fast
-    and safe.
+    All of QList's functionality also applies to QStringList. For
+    example, you can use isEmpty() to test whether the list is empty,
+    and you can call functions like append(), prepend(), insert(),
+    replace(), and remove() to modify a QStringList. In addition,
+    QStringList provides a few convenience functions that make
+    handling lists of strings easier:
 
-    Strings can be added to a list using append(), operator+=(), or
-    operator<<(). For example:
-    \code
-        QStringList fonts;
-        fonts << "Arial" << "Helvetica" << "Times" << "Courier";
-    \endcode
+    \tableofcontents
 
-    To iterate over a string, you can either use index positions or
-    QList's Java-style and STL-style iterator types. Here are
-    examples of each approach.
+    \section1 Adding strings
+
+    Strings can be added to a list using the \l
+    {QList::append()}{append()}, \l
+    {QList::operator+=()}{operator+=()} and \l
+    {QStringList::operator<<()}{operator<<()} functions. For example:
+
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList fonts
+    \printuntil Courier
+
+    \section1 Iterating over the strings
+
+    To iterate over a list, you can either use index positions or
+    QList's Java-style and STL-style iterator types:
 
     Indexing:
-    \code
-        for (int i = 0; i < fonts.size(); ++i)
-            cout << fonts.at(i).ascii() << endl;
-    \endcode
+
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto for (int i = 0; i < fonts.size(); ++i)
+    \printuntil cout << fonts.at(i).toLocal8Bit().constData() << end
 
     Java-style iterator:
-    \code
-        QStringListIterator i(fonts);
-        while (i.hasNext())
-            cout << i.next().ascii() << endl;
-    \endcode
+
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringListIterator
+    \printuntil cout << javaStyleIterator
 
     STL-style iterator:
-    \code
-        QStringList::const_iterator i;
-        for (i = fonts.constBegin(); i != fonts.constEnd(); ++i)
-            cout << (*i).ascii() << endl;
-    \endcode
 
-    QStringListIterator and QMutableStringListIterator are simply
-    typedefs for QListIterator<QString> and
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList::const_iterator
+    \printuntil cout << (*constIterator)
+
+    The QStringListIterator class is simply a type definition for
+    QListIterator<QString>. QStringList also provide the
+    QMutableStringListIterator class which is a type definition for
     QMutableListIterator<QString>.
 
-    You can concatenate all the strings in a string list into a single
-    string (with an optional separator) using join(). For example:
+    \section1 Manipulating the strings
 
-    \code
-        QString str = fonts.join(",");
-        // str == "Arial,Helvetica,Times,Courier"
-    \endcode
+    QStringList provides several functions allowing you to manipulate
+    the contents of a list. You can concatenate all the strings in a
+    string list into a single string (with an optional separator)
+    using the join() function. For example:
 
-    To break up a string into a string list, use QString::split():
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto fonts.join
+    \printuntil // str
 
-    \code
-        QString str = "Arial,Helvetica,Times,Courier";
-        QStringList list = str.split(",");
-        // list: ["Arial", "Helvetica", "Times", "Courier"]
-    \endcode
+    To break up a string into a string list, use the QString::split()
+    function:
+
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList list;
+    \printuntil // list
 
     The argument to split can be a single character, a string, or a
     QRegExp.
 
-    You can sort a string list with sort(), and extract a new list
-    which contains only those strings which contain a particular
-    substring (or match a particular regular expression) using the
-    find() functions. For example:
+    In addition, the \l {QStringList::operator+()}{operator+()}
+    function allows you to concatenate two string lists into one. To
+    sort a string list, use the sort() function.
 
-    \code
-        QStringList monospacedFonts = fonts.find(QRegExp("Courier|Fixed"));
-    \endcode
+    QString list also provides the filter() function which lets you
+    to extract a new list which contains only those strings which
+    contain a particular substring (or match a particular regular
+    expression):
 
-    Similarly, the replace() function calls QString::replace() on
-    each string in the string list in turn. Here's an example that
-    uses it to replace all occurrences of "$QTDIR" with "/usr/lib/qt"
-    in a string list:
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto monospacedFonts
+    \printline monospacedFonts
 
-    \code
-        QStringList files;
-        files << "$QTDIR/src/moc/moc.y"
-              << "$QTDIR/src/moc/moc.l"
-              << "$QTDIR/include/qconfig.h";
+    The contains() function tells you whether the list contains a
+    given string, while the indexOf() function returns the index of
+    the first occurrence of the given string. The lastIndexOf()
+    function on the other hand, returns the index of the last
+    occurrence of the string.
 
-        files.replace("$QTDIR", "/usr/lib/qt");
-    \endcode
+    Finally, the replaceInStrings() function calls QString::replace()
+    on each string in the string list in turn. For example:
 
-    \sa QString, QStringListIterator, QMutableStringListIterator
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList files
+    \printuntil // files
+
+    \sa QString
 */
 
 /*!
@@ -161,23 +171,26 @@
 /*!
     \fn QStringList::QStringList(const QString &str)
 
-    Constructs a string list that contains one string, \a str. Longer
-    lists are easily created like this:
+    Constructs a string list that contains the given string, \a
+    str. Longer lists are easily created like this:
 
-    \code
-        list = (QStringList() << str1 << str2 << str3);
-    \endcode
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto  longerList
+    \printline  longerList
+
+    \sa append()
 */
 
 /*!
     \fn QStringList::QStringList(const QStringList &other)
 
-    Constructs a copy of \a other.
+    Constructs a copy of the \a other string list.
 
-    This operation takes \l{constant time}, because QStringList is
-    \l{implicitly shared}. This makes returning a QStringList from a
-    function very fast. If a shared instance is modified, it will be
-    copied (copy-on-write), and that takes \l{linear time}.
+    This operation takes \l{constant time} because QStringList is
+    \l{implicitly shared}, making the process of returning a
+    QStringList from a function very fast. If a shared instance is
+    modified, it will be copied (copy-on-write), and that takes
+    \l{linear time}.
 
     \sa operator=()
 */
@@ -204,11 +217,11 @@
     which operates in \l{linear-logarithmic time}, i.e. O(\e{n} log \e{n}).
 
     If you want to sort your strings in an arbitrary order, consider
-    using a QMap. For example, you could use a QMap\<QString,
-    QString\> to create a case-insensitive ordering (e.g. with the
-    keys being lower-case versions of the strings, and the values
-    being the strings), or a QMap\<int, QString\> to sort the strings
-    by some integer index.
+    using the QMap class. For example, you could use a QMap<QString,
+    QString> to create a case-insensitive ordering (e.g. with the keys
+    being lower-case versions of the strings, and the values being the
+    strings), or a QMap<int, QString> to sort the strings by some
+    integer index.
 
     \sa qSort()
 */
@@ -283,14 +296,21 @@ void QtPrivate::QStringList_sort(QStringList *that)
     comparison is case sensitive; otherwise the comparison is case
     insensitive.
 
-    \code
-        QStringList list;
-        list << "Bill Murray" << "John Doe" << "Bill Clinton";
-        list = list.filter("Bill");
-        // list: ["Bill Murray", "Bill Clinton"]
-    \endcode
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList list;
+    \printline QStringList list;
+    \skipto list << "Bill Murray"
+    \printuntil  // result
 
-    \sa QString::contains()
+    This is equivalent to
+
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList result;
+    \printline QStringList result;
+    \skipto foreach
+    \printuntil }
+
+    \sa contains()
 */
 QStringList QtPrivate::QStringList_filter(const QStringList *that, const QString &str,
                                           Qt::CaseSensitivity cs)
@@ -307,9 +327,11 @@ QStringList QtPrivate::QStringList_filter(const QStringList *that, const QString
 /*!
     \fn QBool QStringList::contains(const QString &str, Qt::CaseSensitivity cs) const
 
-    Returns true if the list contains the string \a str.
-    Does a case insensitive search if \a cs is Qt::CaseSensitive,
-    otherwise the search will be case insensitive.
+    Returns true if the list contains the string \a str; otherwise
+    returns false. The search is case insensitive if \a cs is
+    Qt::CaseInsensitive; the search is case sensitive by default.
+
+    \sa indexOf(), lastIndexOf(), QString::contains()
  */
 QBool QtPrivate::QStringList_contains(const QStringList *that, const QString &str,
                                       Qt::CaseSensitivity cs)
@@ -350,13 +372,13 @@ QStringList QtPrivate::QStringList_filter(const QStringList *that, const QRegExp
     is found. The \a before text is matched case-sensitively or not
     depending on the \a cs flag.
 
-    Example:
-    \code
-        QStringList list;
-        list << "alpha" << "beta" << "gamma" << "epsilon";
-        list.replace("a", "o");
-        // list == ["olpho", "beto", "gommo", "epsilon"]
-    \endcode
+    For example:
+
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList list;
+    \printline QStringList list;
+    \skipto list << "alpha"
+    \printuntil  // list
 
     \sa QString::replace()
 */
@@ -371,34 +393,33 @@ void QtPrivate::QStringList_replaceInStrings(QStringList *that, const QString &b
 #ifndef QT_NO_REGEXP
 /*!
     \fn QStringList &QStringList::replaceInStrings(const QRegExp &rx, const QString &after)
-
     \overload
 
     Replaces every occurrence of the regexp \a rx, in each of the
     string lists's strings, with \a after. Returns a reference to the
     string list.
 
-    Example:
-    \code
-        QStringList list;
-        list << "alpha" << "beta" << "gamma" << "epsilon";
-        list.replace(QRegExp("^a"), "o");
-        // list == ["olpha", "beta", "gamma", "epsilon"]
-    \endcode
+    For example:
+
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList list;
+    \printline QStringList list;
+    \skipto list << "alpha"
+    \skipto list.clear()
+    \skipto list << "alpha"
+    \printuntil  // list
 
     For regular expressions that contain \l{capturing parentheses},
     occurrences of \bold{\\1}, \bold{\\2}, ..., in \a after are
     replaced with \a{rx}.cap(1), \a{rx}.cap(2), ...
 
-    Example:
-    \code
-        QStringList list;
-        list << "Bill Clinton" << "Murray, Bill";
-        list.replace(QRegExp("^(.*), (.*)$"), "\\2 \\1");
-        // list == ["Bill Clinton", "Bill Murray"]
-    \endcode
+    For example:
 
-    \sa replace()
+    \quotefromfile snippets/qstringlist/main.cpp
+    \skipto QStringList list;
+    \printline QStringList list;
+    \skipto list << "Bill Clinton" << "Murray, Bill"
+    \printuntil  // list
 */
 void QtPrivate::QStringList_replaceInStrings(QStringList *that, const QRegExp &rx, const QString &after)
 {
@@ -408,11 +429,11 @@ void QtPrivate::QStringList_replaceInStrings(QStringList *that, const QRegExp &r
 #endif
 
 /*!
-    \fn QString QStringList::join(const QString &sep) const
+    \fn QString QStringList::join(const QString &separator) const
 
-    Joins the all the string list's strings into a single string with
-    each element separated by the string \a sep (which can be an empty
-    string).
+    Joins all the string list's strings into a single string with each
+    element separated by the the given \a separator (which can be an
+    empty string).
 
     \sa QString::split()
 */
@@ -439,8 +460,8 @@ QString QtPrivate::QStringList_join(const QStringList *that, const QString &sep)
 /*!
     \fn QStringList &QStringList::operator<<(const QString &str)
 
-    Appends string \a str to the string list and returns a reference
-    to the string list.
+    Appends the given string, \a str, to this string list and returns
+    a reference to the string list.
 
     \sa append()
 */
@@ -450,8 +471,8 @@ QString QtPrivate::QStringList_join(const QStringList *that, const QString &sep)
 
     \overload
 
-    Appends \a other to the string list and returns a reference to
-    the string list.
+    Appends the \a other string list to the string list and returns a reference to
+    the latter string list.
 */
 
 #ifndef QT_NO_DATASTREAM
@@ -459,43 +480,44 @@ QString QtPrivate::QStringList_join(const QStringList *that, const QString &sep)
     \fn QDataStream &operator>>(QDataStream &in, QStringList &list)
     \relates QStringList
 
-    Reads a string list from stream \a in into \a list.
+    Reads a string list from the given \a in stream into the specified
+    \a list.
 
-    \sa {Format of the QDataStream operators}
+    \sa {Format of the QDataStream Operators}
 */
 
 /*!
     \fn QDataStream &operator<<(QDataStream &out, const QStringList &list)
     \relates QStringList
 
-    Writes the string list \a list to stream \a out.
+    Writes the given string \a list to the specified \a out stream.
 
-    \sa {Format of the QDataStream operators}
+    \sa {Format of the QDataStream Operators}
 */
 #endif // QT_NO_DATASTREAM
 
 /*!
     \fn QStringList QStringList::grep(const QString &str, bool cs = true) const
 
-    Use find() instead.
+    Use filter() instead.
 */
 
 /*!
     \fn QStringList QStringList::grep(const QRegExp &rx) const
 
-    Use find() instead.
+    Use filter() instead.
 */
 
 /*!
     \fn QStringList &QStringList::gres(const QString &before, const QString &after, bool cs = true)
 
-    Use replace() instead.
+    Use replaceInStrings() instead.
 */
 
 /*!
     \fn QStringList &QStringList::gres(const QRegExp &rx, const QString &after)
 
-    Use replace() instead.
+    Use replaceInStrings() instead.
 */
 
 /*!
@@ -527,13 +549,11 @@ QString QtPrivate::QStringList_join(const QStringList *that, const QString &sep)
 /*!
     \fn int QStringList::indexOf(const QRegExp &rx, int from) const
 
-    \overload
-
     Returns the index position of the first exact match of \a rx in
     the list, searching forward from index position \a from. Returns
     -1 if no item matched.
 
-    \sa lastIndexOf(), QRegExp::exactMatch()
+    \sa lastIndexOf(), contains(), QRegExp::exactMatch()
 */
 int QtPrivate::QStringList_indexOf(const QStringList *that, const QRegExp &rx, int from)
 {
@@ -549,14 +569,12 @@ int QtPrivate::QStringList_indexOf(const QStringList *that, const QRegExp &rx, i
 /*!
     \fn int QStringList::lastIndexOf(const QRegExp &rx, int from) const
 
-    \overload
-
     Returns the index position of the last exact match of \a rx in
     the list, searching backward from index position \a from. If \a
     from is -1 (the default), the search starts at the last item.
     Returns -1 if no item matched.
 
-    \sa indexOf(), QRegExp::exactMatch()
+    \sa indexOf(), contains(), QRegExp::exactMatch()
 */
 int QtPrivate::QStringList_lastIndexOf(const QStringList *that, const QRegExp &rx, int from)
 {
