@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -119,6 +119,19 @@
     \value LastStandardAction
 */
 
+/*!
+    \enum QAccessible::Method
+
+    This enum describes the possible types of methods that can be
+    invoked on an accessible object.
+
+    \value ListSupportedMethods
+    \value SetCursorPosition
+    \value GetCursorPosition
+
+    \sa QAccessibleInterface::invokeMethod()
+*/
+
 /*!
     \enum QAccessible::StateFlag
 
@@ -155,6 +168,7 @@
     \value Traversed        The object is linked and has been visited.
     \value Unavailable      The object is unavailable to the user, e.g. a disabled widget.
     \omitvalue Moveable
+    \omitvalue HasInvokeExtension
 
     Implementations of QAccessibleInterface::state() return a combination
     of these flags.
@@ -351,7 +365,7 @@
     When \a child is 0, the object itself has changed.
 
     Call this function whenever the state of your accessible object or
-    one of it's sub-elements has been changed either programmatically
+    one of its sub-elements has been changed either programmatically
     (e.g. by calling QLabel::setText()) or by user interaction.
 
     If there are no accessibility tools listening to this event, the
@@ -362,7 +376,7 @@
 
 #ifndef QT_NO_LIBRARY
 Q_GLOBAL_STATIC_WITH_ARGS(QFactoryLoader, loader,
-    (QAccessibleFactoryInterface_iid, QCoreApplication::libraryPaths(), "/accessible"))
+    (QAccessibleFactoryInterface_iid, QCoreApplication::libraryPaths(), QLatin1String("/accessible")))
 #endif
 
 Q_GLOBAL_STATIC(QList<QAccessible::InterfaceFactory>, qAccessibleFactories);
@@ -635,7 +649,7 @@ bool QAccessible::isActive()
     children are an integral part of the object itself, for example, the
     different sections in a scrollbar.
 
-    If an accessible object provides information about it's children
+    If an accessible object provides information about its children
     through one QAccessibleInterface, the children are referenced
     using indexes. The index is 1-based for the children, i.e. 0
     refers to the object itself, 1 to the first child, 2 to the second
@@ -673,7 +687,7 @@ bool QAccessible::isActive()
     \fn int QAccessibleInterface::childCount() const
 
     Returns the number of children that belong to this object. A child
-    can provide accessibility information on it's own (e.g. a child
+    can provide accessibility information on its own (e.g. a child
     widget), or be a sub-element of this accessible object.
 
     All objects provide this information.
@@ -962,4 +976,27 @@ const QAccessibleInterface *other, int otherChild) const
     \sa value()
 */
 
+/*!
+    \since 4.2
+
+    Invokes a \a method on \a child with the given parameters \a params
+    and returns the result of the operation as QVariant.
+
+    Note that the type of the returned QVariant depends on the action.
+
+    Returns an invalid QVariant if the object doesn't support the action.
+*/
+QVariant QAccessibleInterface::invokeMethod(Method method, int child, const QVariantList &params)
+{
+    if (!(state(0) & HasInvokeExtension))
+        return QVariant();
+
+    return static_cast<QAccessibleInterfaceEx *>(this)->invokeMethodEx(method, child, params);
+}
+
+QVariant QAccessibleInterfaceEx::virtual_hook(const QVariant &)
+{
+    return QVariant();
+}
+
 #endif
