@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -28,6 +28,8 @@
 #include <QtCore/qstring.h>
 #include <QtGui/qdialog.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_FILEDIALOG
@@ -66,7 +68,7 @@ public:
     };
     Q_DECLARE_FLAGS(Options, Option)
 
-    QFileDialog(QWidget *parent, Qt::WFlags f);
+    QFileDialog(QWidget *parent, Qt::WindowFlags f);
     explicit QFileDialog(QWidget *parent = 0,
                          const QString &caption = QString(),
                          const QString &directory = QString(),
@@ -212,29 +214,29 @@ protected:
 private:
     Q_DECLARE_PRIVATE(QFileDialog)
     Q_DISABLE_COPY(QFileDialog)
-    Q_PRIVATE_SLOT(d_func(), void reload())
-    Q_PRIVATE_SLOT(d_func(), void navigateToPrevious())
-    Q_PRIVATE_SLOT(d_func(), void navigateToParent())
-    Q_PRIVATE_SLOT(d_func(), void enterDirectory(const QModelIndex &index))
-    Q_PRIVATE_SLOT(d_func(), void enterDirectory(const QString &path))
-    Q_PRIVATE_SLOT(d_func(), void enterDirectory())
-    Q_PRIVATE_SLOT(d_func(), void showList())
-    Q_PRIVATE_SLOT(d_func(), void showDetails())
-    Q_PRIVATE_SLOT(d_func(), void showHidden())
-    Q_PRIVATE_SLOT(d_func(), void useFilter(const QString &filter))
-    Q_PRIVATE_SLOT(d_func(), void updateFileName(const QItemSelection &selection))
-    Q_PRIVATE_SLOT(d_func(), void autoCompleteFileName(const QString &text))
-    Q_PRIVATE_SLOT(d_func(), void autoCompleteDirectory(const QString &text))
-    Q_PRIVATE_SLOT(d_func(), void showContextMenu(const QPoint &pos))
-    Q_PRIVATE_SLOT(d_func(), void createDirectory())
-    Q_PRIVATE_SLOT(d_func(), void renameCurrent())
-    Q_PRIVATE_SLOT(d_func(), void deleteCurrent())
-    Q_PRIVATE_SLOT(d_func(), void sortByName())
-    Q_PRIVATE_SLOT(d_func(), void sortBySize())
-    Q_PRIVATE_SLOT(d_func(), void sortByDate())
-    Q_PRIVATE_SLOT(d_func(), void setUnsorted())
-    Q_PRIVATE_SLOT(d_func(), void sortByColumn(int))
-    Q_PRIVATE_SLOT(d_func(), void currentChanged(const QModelIndex &index))
+    Q_PRIVATE_SLOT(d_func(), void _q_reload())
+    Q_PRIVATE_SLOT(d_func(), void _q_navigateToPrevious())
+    Q_PRIVATE_SLOT(d_func(), void _q_navigateToParent())
+    Q_PRIVATE_SLOT(d_func(), void _q_enterDirectory(const QModelIndex &index))
+    Q_PRIVATE_SLOT(d_func(), void _q_enterDirectory(const QString &path))
+    Q_PRIVATE_SLOT(d_func(), void _q_enterDirectory())
+    Q_PRIVATE_SLOT(d_func(), void _q_showList())
+    Q_PRIVATE_SLOT(d_func(), void _q_showDetails())
+    Q_PRIVATE_SLOT(d_func(), void _q_showHidden())
+    Q_PRIVATE_SLOT(d_func(), void _q_useFilter(const QString &filter))
+    Q_PRIVATE_SLOT(d_func(), void _q_updateFileName(const QItemSelection &selection))
+    Q_PRIVATE_SLOT(d_func(), void _q_autoCompleteFileName(const QString &text))
+    Q_PRIVATE_SLOT(d_func(), void _q_autoCompleteDirectory(const QString &text))
+    Q_PRIVATE_SLOT(d_func(), void _q_showContextMenu(const QPoint &pos))
+    Q_PRIVATE_SLOT(d_func(), void _q_createDirectory())
+    Q_PRIVATE_SLOT(d_func(), void _q_renameCurrent())
+    Q_PRIVATE_SLOT(d_func(), void _q_deleteCurrent())
+    Q_PRIVATE_SLOT(d_func(), void _q_sortByName())
+    Q_PRIVATE_SLOT(d_func(), void _q_sortBySize())
+    Q_PRIVATE_SLOT(d_func(), void _q_sortByDate())
+    Q_PRIVATE_SLOT(d_func(), void _q_setUnsorted())
+    Q_PRIVATE_SLOT(d_func(), void _q_sortByColumn(int))
+    Q_PRIVATE_SLOT(d_func(), void _q_currentChanged(const QModelIndex &index))
 };
 
 inline void QFileDialog::setDirectory(const QDir &adirectory)
@@ -244,4 +246,6 @@ Q_DECLARE_OPERATORS_FOR_FLAGS(QFileDialog::Options)
 
 #endif // QT_NO_FILEDIALOG
 
+QT_END_HEADER
+
 #endif // QFILEDIALOG_H
