@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -27,7 +27,7 @@
 **
 ** Copyright (C) 2003-2004 immodule for Qt Project.  All rights reserved.
 **
-** This file is written to contribute to Trolltech AS under their own
+** This file is written to contribute to Trolltech ASA under their own
 ** licence. You may use this file under your Qt license. Following
 ** description is copied from their original file headers. Contact
 ** immodule-qt@freedesktop.org if any conditions of this licensing are
@@ -42,6 +42,8 @@
 #include <QtCore/qfactoryinterface.h>
 #include <QtCore/qstringlist.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #if !defined(QT_NO_IM) && !defined(QT_NO_LIBRARY)
@@ -77,4 +79,6 @@ public:
 
 #endif // QT_NO_IM
 
+QT_END_HEADER
+
 #endif // QINPUTCONTEXTPLUGIN_H
