@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -78,7 +78,7 @@ struct QCursorData {
     Cursor hcurs;
     Pixmap pm, pmm;
 #elif defined (Q_WS_MAC)
-    enum { TYPE_None, TYPE_CursorImage, TYPE_CursPtr, TYPE_ThemeCursor, TYPE_FakeCursor, TYPE_BigCursor } type;
+    enum { TYPE_None, TYPE_CursPtr, TYPE_ThemeCursor, TYPE_FakeCursor, TYPE_BigCursor } type;
     union {
         struct {
             uint my_cursor:1;
@@ -93,7 +93,6 @@ struct QCursorData {
 #ifdef QMAC_USE_BIG_CURSOR_API
         char *big_cursor_name;
 #endif
-        CursorImageRec *ci;
         struct {
             QMacAnimateCursor *anim;
             ThemeCursor curs;
