@@ -246,11 +246,10 @@ void QGridLayoutPrivate::recalcHFW(int w, int spacing)
     int mh = 0;
     int n = 0;
     for (int r = 0; r < rr; r++) {
-        if (!rData[r].empty) {
-            h += rData[r].sizeHint;
-            mh += rData[r].minimumSize;
+        h += rData[r].sizeHint;
+        mh += rData[r].minimumSize;
+        if (!rData[r].empty)
             n++;
-        }
     }
     if (n) {
         h += (n - 1) * spacing;
@@ -268,8 +267,8 @@ int QGridLayoutPrivate::heightForWidth(int w, int margin, int spacing)
     if (!has_hfw)
         return -1;
     if (w + 2*margin != hfw_width) {
-        qGeomCalc(colData, 0, cc, 0, w+2*margin, spacing);
-        recalcHFW(w+2*margin, spacing);
+        qGeomCalc(colData, 0, cc, 0, w - 2*margin, spacing);
+        recalcHFW(w - 2*margin, spacing);
     }
     return hfw_height + 2*margin;
 }
@@ -289,20 +288,20 @@ QSize QGridLayoutPrivate::findSize(int QLayoutStruct::*size, int spacer) const
     int h = 0;
     int n = 0;
 
-    for (int r = 0; r < rr; r++)
-        if (!rowData[r].empty) {
-            h = h + rowData[r].*size;
+    for (int r = 0; r < rr; r++) {
+        h = h + rowData[r].*size;
+        if (!rowData[r].empty)
             n++;
-        }
+    }
     if (n)
         h += (n - 1) * spacer;
 
     n = 0;
-    for (int c = 0; c < cc; c++)
-        if (!colData[c].empty) {
-            w = w + colData[c].*size;
+    for (int c = 0; c < cc; c++) {
+        w = w + colData[c].*size;
+        if (!colData[c].empty)
             n++;
-        }
+    }
     if (n)
         w += (n - 1) * spacer;
 
@@ -357,6 +356,9 @@ void QGridLayoutPrivate::setSize(int r, int c)
         rSpacing.resize(newR);
         for (int i = rr; i < newR; i++) {
             rowData[i].init();
+            rowData[i].maximumSize = 0;
+            rowData[i].pos = 0;
+            rowData[i].size = 0;
             rStretch[i] = 0;
             rSpacing[i] = 0;
         }
@@ -368,6 +370,9 @@ void QGridLayoutPrivate::setSize(int r, int c)
         cSpacing.resize(newC);
         for (int i = cc; i < newC; i++) {
             colData[i].init();
+            colData[i].maximumSize = 0;
+            colData[i].pos = 0;
+            colData[i].size = 0;
             cStretch[i] = 0;
             cSpacing[i] = 0;
         }
@@ -460,9 +465,8 @@ void QGridLayoutPrivate::addData(QGridBox *box, bool r, bool c)
         colData[box->col].minimumSize = qMax(minS.width(),
                                               colData[box->col].minimumSize);
 
-        qMaxExpCalc(colData[box->col].maximumSize, colData[box->col].expansive,
-                     maxS.width(),
-                     box->expandingDirections() & Qt::Horizontal);
+        qMaxExpCalc(colData[box->col].maximumSize, colData[box->col].expansive, colData[box->col].empty,
+                    maxS.width(), box->expandingDirections() & Qt::Horizontal, box->isEmpty());
     }
     if (r) {
         if (!rStretch[box->row])
@@ -473,17 +477,8 @@ void QGridLayoutPrivate::addData(QGridBox *box, bool r, bool c)
         rowData[box->row].minimumSize = qMax(minS.height(),
                                               rowData[box->row].minimumSize);
 
-        qMaxExpCalc(rowData[box->row].maximumSize, rowData[box->row].expansive,
-                     maxS.height(),
-                     box->expandingDirections() & Qt::Vertical);
-    }
-    if (!box->isEmpty()) {
-        // Empty boxes (i.e. spacers) do not get borders. This is
-        // hacky, but compatible.
-        if (c)
-            colData[box->col].empty = false;
-        if (r)
-            rowData[box->row].empty = false;
+        qMaxExpCalc(rowData[box->row].maximumSize, rowData[box->row].expansive, rowData[box->row].empty,
+                    maxS.height(), box->expandingDirections() & Qt::Vertical, box->isEmpty());
     }
 }
 
@@ -497,6 +492,8 @@ static void distributeMultiBox(QVector<QLayoutStruct> &chain, int spacing, int s
     for (i = start; i <= end; i++) {
         w += chain[i].minimumSize;
         wh += chain[i].sizeHint;
+        if (chain[i].empty)
+            chain[i].maximumSize = QWIDGETSIZE_MAX;
         max += chain[i].maximumSize;
         chain[i].empty = false;
         if (stretchArray[i] == 0)
@@ -555,11 +552,14 @@ void QGridLayoutPrivate::setupLayoutData(int spacing)
     has_hfw = false;
     int i;
 
-    for (i = 0; i < rr; i++)
+    for (i = 0; i < rr; i++) {
         rowData[i].init(rStretch[i], rSpacing[i]);
-    for (i = 0; i < cc; i++)
+        rowData[i].maximumSize = rStretch[i] ? QLAYOUTSIZE_MAX : rSpacing[i];
+    }
+    for (i = 0; i < cc; i++) {
         colData[i].init(cStretch[i], cSpacing[i]);
-
+        colData[i].maximumSize = cStretch[i] ? QLAYOUTSIZE_MAX : cSpacing[i];
+    }
     for (int pass = 0; pass < 2; ++pass) {
         for (i = 0; i < things.size(); ++i) {
             QGridBox *box = things.at(i);
@@ -726,7 +726,7 @@ QRect QGridLayoutPrivate::cellRect(int row, int col) const
         return QRect();
 
     const QVector<QLayoutStruct> *rDataPtr;
-    if (has_hfw)
+    if (has_hfw && hfwData)
         rDataPtr = hfwData;
     else
         rDataPtr = &rowData;
@@ -805,7 +805,7 @@ QRect QGridLayoutPrivate::cellRect(int row, int col) const
     the margin width for a top-level layout, or to the same as the
     parent layout.
 
-    \sa QBoxLayout, QStackedLayout, {Layout Classes}
+    \sa QBoxLayout, QStackedLayout, {Layout Classes}, {Basic Layouts Example}
 */
 
 
@@ -1290,6 +1290,7 @@ void QGridLayout::setRowStretch(int row, int stretch)
 {
     Q_D(QGridLayout);
     d->setRowStretch(row, stretch);
+    invalidate();
 }
 
 /*!
@@ -1335,6 +1336,7 @@ void QGridLayout::setColumnStretch(int column, int stretch)
 {
     Q_D(QGridLayout);
     d->setColStretch(column, stretch);
+    invalidate();
 }
 
 
@@ -1348,6 +1350,7 @@ void QGridLayout::setRowMinimumHeight(int row, int minSize)
 {
     Q_D(QGridLayout);
     d->setRowSpacing(row, minSize);
+    invalidate();
 }
 
 /*!
@@ -1370,6 +1373,7 @@ void QGridLayout::setColumnMinimumWidth(int column, int minSize)
 {
     Q_D(QGridLayout);
     d->setColSpacing(column, minSize);
+    invalidate();
 }
 
 /*!
