@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -21,18 +21,18 @@
 **
 ****************************************************************************/
 
+#include "qdebug.h"
 #include "qopentype_p.h"
 #include "qfontengine_p.h"
 #include "qscriptengine_p.h"
 
-#ifndef QT_NO_FREETYPE
+#ifndef QT_NO_OPENTYPE
 //  --------------------------------------------------------------------------------------------------------------------
 // Open type support
 //  --------------------------------------------------------------------------------------------------------------------
 
 //#define OT_DEBUG
 
-#ifdef OT_DEBUG
 static inline char *tag_to_string(FT_ULong tag)
 {
     static char string[5];
@@ -43,6 +43,16 @@ static inline char *tag_to_string(FT_ULong tag)
     string[4] = 0;
     return string;
 }
+#ifdef OT_DEBUG
+static void dump_string(HB_Buffer buffer)
+{
+    for (uint i = 0; i < buffer->in_length; ++i) {
+        qDebug("    %x: cluster=%d", buffer->in_string[i].gindex, buffer->in_string[i].cluster);
+    }
+}
+#define DEBUG qDebug
+#else
+#define DEBUG if (1) ; else qDebug
 #endif
 
 #define DefaultLangSys 0xffff
@@ -59,7 +69,13 @@ struct OTScripts {
 };
 static const OTScripts ot_scripts [] = {
     // Common
-    { FT_MAKE_TAG('D', 'F', 'L', 'T'), 0 },
+    { FT_MAKE_TAG('l', 'a', 't', 'n'), 0 },
+    // Greek
+    { FT_MAKE_TAG('g', 'r', 'e', 'k'), 0 },
+    // Cyrillic
+    { FT_MAKE_TAG('c', 'y', 'r', 'l'), 0 },
+    // Armenian
+    { FT_MAKE_TAG('a', 'r', 'm', 'n'), 0 },
     // Hebrew
     { FT_MAKE_TAG('h', 'e', 'b', 'r'), 1 },
     // Arabic
@@ -87,7 +103,6 @@ static const OTScripts ot_scripts [] = {
     // Malayalam
     { FT_MAKE_TAG('m', 'l', 'y', 'm'), 1 },
     // Sinhala
-    // ### could not find any OT specs on this
     { FT_MAKE_TAG('s', 'i', 'n', 'h'), 1 },
     // Thai
     { FT_MAKE_TAG('t', 'h', 'a', 'i'), 1 },
@@ -97,45 +112,51 @@ static const OTScripts ot_scripts [] = {
     { FT_MAKE_TAG('t', 'i', 'b', 't'), 1 },
     // Myanmar
     { FT_MAKE_TAG('m', 'y', 'm', 'r'), 1 },
+    // Georgian
+    { FT_MAKE_TAG('g', 'e', 'o', 'r'), 0 },
     // Hangul
     { FT_MAKE_TAG('h', 'a', 'n', 'g'), 1 },
+    // Ogham
+    { FT_MAKE_TAG('o', 'g', 'a', 'm'), 0 },
+    // Runic
+    { FT_MAKE_TAG('r', 'u', 'n', 'r'), 0 },
     // Khmer
-    { FT_MAKE_TAG('k', 'h', 'm', 'r'), 1 },
+    { FT_MAKE_TAG('k', 'h', 'm', 'r'), 1 }
 };
+enum { NumOTScripts = sizeof(ot_scripts)/sizeof(OTScripts) };
 
 QOpenType::QOpenType(QFontEngine *fe, FT_Face _face)
-    : fontEngine(fe), face(_face), gdef(0), gsub(0), gpos(0), current_script(0)
+    : fontEngine(fe), face(_face), gdef(0), gsub(0), gpos(0), current_script(0xffffffff)
 {
-    otl_buffer_new(face->memory, &otl_buffer);
+    Q_ASSERT(NumOTScripts == (int)QUnicodeTables::ScriptCount);
+
+    hb_buffer_new(face->memory, &hb_buffer);
     tmpAttributes = 0;
     tmpLogClusters = 0;
 
+    kerning_feature_selected = false;
+
     FT_Error error;
-    if ((error = TT_Load_GDEF_Table(face, &gdef))) {
-#ifdef OT_DEBUG
-        qDebug("error loading gdef table: %d", error);
-#endif
+    if ((error = HB_Load_GDEF_Table(face, &gdef))) {
+        DEBUG("error loading gdef table: %d", error);
         gdef = 0;
     }
 
-    if ((error = TT_Load_GSUB_Table(face, &gsub, gdef))) {
+    DEBUG() << "trying to load gsub table";
+    if ((error = HB_Load_GSUB_Table(face, &gsub, gdef))) {
         gsub = 0;
-#ifdef OT_DEBUG
         if (error != FT_Err_Table_Missing) {
-            qDebug("error loading gsub table: %d", error);
+            DEBUG("error loading gsub table: %d", error);
         } else {
-            qDebug("face doesn't have a gsub table");
+            DEBUG("face doesn't have a gsub table");
         }
-#endif
     }
 
-    if ((error = TT_Load_GPOS_Table(face, &gpos, gdef))) {
+    if ((error = HB_Load_GPOS_Table(face, &gpos, gdef))) {
         gpos = 0;
-#ifdef OT_DEBUG
-        qDebug("error loading gpos table: %d", error);
-#endif
+        DEBUG("error loading gpos table: %d", error);
     }
-    
+
     for (uint i = 0; i < QUnicodeTables::ScriptCount; ++i)
         supported_scripts[i] = checkScript(i);
 }
@@ -143,13 +164,13 @@ QOpenType::QOpenType(QFontEngine *fe, FT_Face _face)
 QOpenType::~QOpenType()
 {
     if (gpos)
-        TT_Done_GPOS_Table(gpos);
+        HB_Done_GPOS_Table(gpos);
     if (gsub)
-        TT_Done_GSUB_Table(gsub);
+        HB_Done_GSUB_Table(gsub);
     if (gdef)
-        TT_Done_GDEF_Table(gdef);
-    if (otl_buffer)
-        otl_buffer_free(otl_buffer);
+        HB_Done_GDEF_Table(gdef);
+    if (hb_buffer)
+        hb_buffer_free(hb_buffer);
     if (tmpAttributes)
         free(tmpAttributes);
     if (tmpLogClusters)
@@ -166,28 +187,28 @@ bool QOpenType::checkScript(unsigned int script)
     if (requirements & RequiresGsub) {
         if (!gsub)
             return false;
-        
+
         FT_UShort script_index;
-        FT_Error error = TT_GSUB_Select_Script(gsub, tag, &script_index);
+        FT_Error error = HB_GSUB_Select_Script(gsub, tag, &script_index);
         if (error) {
-#ifdef OT_DEBUG
-            qDebug("could not select script %d in GSub table: %d", (int)script, error);
-#endif
-            return false;
+            DEBUG("could not select script %d in GSub table: %d", (int)script, error);
+            error = HB_GSUB_Select_Script(gsub, FT_MAKE_TAG('D', 'F', 'L', 'T'), &script_index);
+            if (error)
+                return false;
         }
     }
-    
+
     if (requirements & RequiresGpos) {
         if (!gpos)
             return false;
-        
+
         FT_UShort script_index;
-        FT_Error error = TT_GPOS_Select_Script(gpos, script, &script_index);
+        FT_Error error = HB_GPOS_Select_Script(gpos, script, &script_index);
         if (error) {
-#ifdef OT_DEBUG
-            qDebug("could not select script in gpos table: %d", error);
-#endif
-            return false;
+            DEBUG("could not select script in gpos table: %d", error);
+            error = HB_GPOS_Select_Script(gpos, FT_MAKE_TAG('D', 'F', 'L', 'T'), &script_index);
+            if (error)
+                return false;
         }
 
     }
@@ -195,11 +216,11 @@ bool QOpenType::checkScript(unsigned int script)
 }
 
 
-void QOpenType::selectScript(unsigned int script, const Features *features)
+void QOpenType::selectScript(QShaperItem *item, unsigned int script, const Features *features)
 {
-    if (current_script == script)
+    if (current_script == script && kerning_feature_selected == item->kerning_enabled)
         return;
-    
+
     assert(script < QUnicodeTables::ScriptCount);
     // find script in our list of supported scripts.
     uint tag = ot_scripts[script].tag;
@@ -207,64 +228,73 @@ void QOpenType::selectScript(unsigned int script, const Features *features)
     if (gsub && features) {
 #ifdef OT_DEBUG
         {
-            TTO_FeatureList featurelist = gsub->FeatureList;
+            HB_FeatureList featurelist = gsub->FeatureList;
             int numfeatures = featurelist.FeatureCount;
-            qDebug("gsub table has %d features", numfeatures);
-            for(int i = 0; i < numfeatures; i++) {
-                TTO_FeatureRecord *r = featurelist.FeatureRecord + i;
-                qDebug("   feature '%s'", tag_to_string(r->FeatureTag));
+            DEBUG("gsub table has %d features", numfeatures);
+            for (int i = 0; i < numfeatures; i++) {
+                HB_FeatureRecord *r = featurelist.FeatureRecord + i;
+                DEBUG("   feature '%s'", tag_to_string(r->FeatureTag));
             }
         }
 #endif
-        TT_GSUB_Clear_Features(gsub);
+        HB_GSUB_Clear_Features(gsub);
         FT_UShort script_index;
-        FT_Error error = TT_GSUB_Select_Script(gsub, tag, &script_index);
+        FT_Error error = HB_GSUB_Select_Script(gsub, tag, &script_index);
         if (!error) {
-#ifdef OT_DEBUG
-            qDebug("script %s has script index %d", tag_to_string(script), script_index);
-#endif
+            DEBUG("script %s has script index %d", tag_to_string(script), script_index);
             while (features->tag) {
                 FT_UShort feature_index;
-                error = TT_GSUB_Select_Feature(gsub, features->tag, script_index, 0xffff, &feature_index);
+                error = HB_GSUB_Select_Feature(gsub, features->tag, script_index, 0xffff, &feature_index);
                 if (!error) {
-#ifdef OT_DEBUG
-                    qDebug("  adding feature %s", tag_to_string(features->tag));
-#endif
-                    TT_GSUB_Add_Feature(gsub, feature_index, features->property);
+                    DEBUG("  adding feature %s", tag_to_string(features->tag));
+                    HB_GSUB_Add_Feature(gsub, feature_index, features->property);
                 }
                 ++features;
             }
         }
     }
-    
+
+    // reset
+    kerning_feature_selected = false;
+
     if (gpos) {
-        TT_GPOS_Clear_Features(gpos);
+        HB_GPOS_Clear_Features(gpos);
         FT_UShort script_index;
-        FT_Error error = TT_GPOS_Select_Script(gpos, tag, &script_index);
+        FT_Error error = HB_GPOS_Select_Script(gpos, tag, &script_index);
         if (!error) {
 #ifdef OT_DEBUG
             {
-                TTO_FeatureList featurelist = gpos->FeatureList;
+                HB_FeatureList featurelist = gpos->FeatureList;
                 int numfeatures = featurelist.FeatureCount;
-                qDebug("gpos table has %d features", numfeatures);
+                DEBUG("gpos table has %d features", numfeatures);
                 for(int i = 0; i < numfeatures; i++) {
-                    TTO_FeatureRecord *r = featurelist.FeatureRecord + i;
+                    HB_FeatureRecord *r = featurelist.FeatureRecord + i;
                     FT_UShort feature_index;
-                    TT_GPOS_Select_Feature(gpos, r->FeatureTag, script_index, 0xffff, &feature_index);
-                    qDebug("   feature '%s'", tag_to_string(r->FeatureTag));
+                    HB_GPOS_Select_Feature(gpos, r->FeatureTag, script_index, 0xffff, &feature_index);
+                    DEBUG("   feature '%s'", tag_to_string(r->FeatureTag));
                 }
             }
 #endif
-            FT_ULong *feature_tag_list;
-            error = TT_GPOS_Query_Features(gpos, script_index, 0xffff, &feature_tag_list);
+            FT_ULong *feature_tag_list_buffer;
+            error = HB_GPOS_Query_Features(gpos, script_index, 0xffff, &feature_tag_list_buffer);
             if (!error) {
+                FT_ULong *feature_tag_list = feature_tag_list_buffer;
                 while (*feature_tag_list) {
                     FT_UShort feature_index;
-                    error = TT_GPOS_Select_Feature(gpos, *feature_tag_list, script_index, 0xffff, &feature_index);
+                    if (*feature_tag_list == FT_MAKE_TAG('k', 'e', 'r', 'n')) {
+                        if (!item->kerning_enabled) {
+                            ++feature_tag_list;
+                            continue;
+                        }
+                        kerning_feature_selected = true;
+                    }
+                    error = HB_GPOS_Select_Feature(gpos, *feature_tag_list, script_index, 0xffff, &feature_index);
                     if (!error)
-                        TT_GPOS_Add_Feature(gpos, feature_index, PositioningProperties);
+                        HB_GPOS_Add_Feature(gpos, feature_index, PositioningProperties);
                     ++feature_tag_list;
                 }
+                FT_Memory memory = gpos->memory;
+                FREE(feature_tag_list_buffer);
             }
         }
     }
@@ -272,98 +302,95 @@ void QOpenType::selectScript(unsigned int script, const Features *features)
     current_script = script;
 }
 
-#ifdef OT_DEBUG
-static void dump_string(OTL_Buffer buffer)
-{
-    for (uint i = 0; i < buffer->in_length; ++i) {
-        qDebug("    %x: cluster=%d", buffer->in_string[i].gindex, buffer->in_string[i].cluster);
-    }
-}
-#endif
 
 extern void qt_heuristicPosition(QShaperItem *item);
 
-void QOpenType::shape(QShaperItem *item, const unsigned int *properties)
+bool QOpenType::shape(QShaperItem *item, const unsigned int *properties)
 {
     length = item->num_glyphs;
 
-    otl_buffer_clear(otl_buffer);
+    hb_buffer_clear(hb_buffer);
 
     tmpAttributes = (QGlyphLayout::Attributes *) realloc(tmpAttributes, length*sizeof(QGlyphLayout::Attributes));
     tmpLogClusters = (unsigned int *) realloc(tmpLogClusters, length*sizeof(unsigned int));
     for (int i = 0; i < length; ++i) {
-        otl_buffer_add_glyph(otl_buffer, item->glyphs[i].glyph, properties ? properties[i] : 0, i);
+        hb_buffer_add_glyph(hb_buffer, item->glyphs[i].glyph, properties ? properties[i] : 0, i);
         tmpAttributes[i] = item->glyphs[i].attributes;
         tmpLogClusters[i] = item->log_clusters[i];
     }
 
 #ifdef OT_DEBUG
-    qDebug("-----------------------------------------");
-//     qDebug("log clusters before shaping:");
+    DEBUG("-----------------------------------------");
+//     DEBUG("log clusters before shaping:");
 //     for (int j = 0; j < length; j++)
-//         qDebug("    log[%d] = %d", j, item->log_clusters[j]);
-    qDebug("original glyphs: %p", item->glyphs);
+//         DEBUG("    log[%d] = %d", j, item->log_clusters[j]);
+    DEBUG("original glyphs: %p", item->glyphs);
     for (int i = 0; i < length; ++i)
-        qDebug("   glyph=%4x", otl_buffer->in_string[i].gindex);
-//     dump_string(otl_buffer);
+        DEBUG("   glyph=%4x", hb_buffer->in_string[i].gindex);
+//     dump_string(hb_buffer);
 #endif
 
     loadFlags = item->flags & QTextEngine::DesignMetrics ? FT_LOAD_NO_HINTING : FT_LOAD_DEFAULT;
 
-    if (gsub)
-        TT_GSUB_Apply_String(gsub, otl_buffer);
+    if (gsub) {
+        uint error = HB_GSUB_Apply_String(gsub, hb_buffer);
+        if (error && error != HB_Err_Not_Covered)
+            return false;
+    }
 
 #ifdef OT_DEBUG
-//     qDebug("log clusters before shaping:");
+//     DEBUG("log clusters before shaping:");
 //     for (int j = 0; j < length; j++)
-//         qDebug("    log[%d] = %d", j, item->log_clusters[j]);
-    qDebug("shaped glyphs:");
+//         DEBUG("    log[%d] = %d", j, item->log_clusters[j]);
+    DEBUG("shaped glyphs:");
     for (int i = 0; i < length; ++i)
-        qDebug("   glyph=%4x", otl_buffer->in_string[i].gindex);
-    qDebug("-----------------------------------------");
-//     dump_string(otl_buffer);
+        DEBUG("   glyph=%4x", hb_buffer->in_string[i].gindex);
+    DEBUG("-----------------------------------------");
+//     dump_string(hb_buffer);
 #endif
+
+    return true;
 }
 
-bool QOpenType::positionAndAdd(QShaperItem *item, bool doLogClusters)
+bool QOpenType::positionAndAdd(QShaperItem *item, int availableGlyphs, bool doLogClusters)
 {
     if (gpos) {
 #ifdef Q_WS_X11
         Q_ASSERT(fontEngine->type() == QFontEngine::Freetype);
         face = static_cast<QFontEngineFT *>(fontEngine)->lockFace();
 #endif
-        memset(otl_buffer->positions, 0, otl_buffer->in_length*sizeof(OTL_PositionRec));
+        memset(hb_buffer->positions, 0, hb_buffer->in_length*sizeof(HB_PositionRec));
         // #### check that passing "false,false" is correct
-        TT_GPOS_Apply_String(face, gpos, loadFlags, otl_buffer, false, false);
+        HB_GPOS_Apply_String(face, gpos, loadFlags, hb_buffer, false, false);
 #ifdef Q_WS_X11
         static_cast<QFontEngineFT *>(fontEngine)->unlockFace();
 #endif
     }
-    
+
     // make sure we have enough space to write everything back
-    if (item->num_glyphs < (int)otl_buffer->in_length) {
-        item->num_glyphs = otl_buffer->in_length;
+    if (availableGlyphs < (int)hb_buffer->in_length) {
+        item->num_glyphs = hb_buffer->in_length;
         return false;
     }
 
     QGlyphLayout *glyphs = item->glyphs;
 
-    for (unsigned int i = 0; i < otl_buffer->in_length; ++i) {
-        glyphs[i].glyph = otl_buffer->in_string[i].gindex;
-        glyphs[i].attributes = tmpAttributes[otl_buffer->in_string[i].cluster];
-        if (i && otl_buffer->in_string[i].cluster == otl_buffer->in_string[i-1].cluster)
+    for (unsigned int i = 0; i < hb_buffer->in_length; ++i) {
+        glyphs[i].glyph = hb_buffer->in_string[i].gindex;
+        glyphs[i].attributes = tmpAttributes[hb_buffer->in_string[i].cluster];
+        if (i && hb_buffer->in_string[i].cluster == hb_buffer->in_string[i-1].cluster)
             glyphs[i].attributes.clusterStart = false;
     }
-    item->num_glyphs = otl_buffer->in_length;
+    item->num_glyphs = hb_buffer->in_length;
 
     if (doLogClusters) {
         // we can't do this for indic, as we pass the stuf in syllables and it's easier to do it in the shaper.
         unsigned short *logClusters = item->log_clusters;
         int clusterStart = 0;
         int oldCi = 0;
-        for (unsigned int i = 0; i < otl_buffer->in_length; ++i) {
-            int ci = otl_buffer->in_string[i].cluster;
-            //         qDebug("   ci[%d] = %d mark=%d, cmb=%d, cs=%d",
+        for (unsigned int i = 0; i < hb_buffer->in_length; ++i) {
+            int ci = hb_buffer->in_string[i].cluster;
+            //         DEBUG("   ci[%d] = %d mark=%d, cmb=%d, cs=%d",
             //                i, ci, glyphAttributes[i].mark, glyphAttributes[i].combiningClass, glyphAttributes[i].clusterStart);
             if (!glyphs[i].attributes.mark && glyphs[i].attributes.clusterStart && ci != oldCi) {
                 for (int j = oldCi; j < ci; j++)
@@ -377,16 +404,16 @@ bool QOpenType::positionAndAdd(QShaperItem *item, bool doLogClusters)
     }
 
     // calulate the advances for the shaped glyphs
-//     qDebug("unpositioned: ");
+//     DEBUG("unpositioned: ");
     item->font->recalcAdvances(item->num_glyphs, glyphs, QFlag(item->flags));
 
     // positioning code:
     if (gpos) {
-        OTL_Position positions = otl_buffer->positions;
+        HB_Position positions = hb_buffer->positions;
 
-//         qDebug("positioned glyphs:");
-        for (unsigned int i = 0; i < otl_buffer->in_length; i++) {
-//             qDebug("    %d:\t orig advance: (%d/%d)\tadv=(%d/%d)\tpos=(%d/%d)\tback=%d\tnew_advance=%d", i,
+//         DEBUG("positioned glyphs:");
+        for (unsigned int i = 0; i < hb_buffer->in_length; i++) {
+//             DEBUG("    %d:\t orig advance: (%d/%d)\tadv=(%d/%d)\tpos=(%d/%d)\tback=%d\tnew_advance=%d", i,
 //                    glyphs[i].advance.x.toInt(), glyphs[i].advance.y.toInt(),
 //                    (int)(positions[i].x_advance >> 6), (int)(positions[i].y_advance >> 6),
 //                    (int)(positions[i].x_pos >> 6), (int)(positions[i].y_pos >> 6),
@@ -416,27 +443,28 @@ bool QOpenType::positionAndAdd(QShaperItem *item, bool doLogClusters)
                     --back;
                 }
             }
-//             qDebug("   ->\tadv=%d\tpos=(%d/%d)",
+//             DEBUG("   ->\tadv=%d\tpos=(%d/%d)",
 //                    glyphs[i].advance.x.toInt(), glyphs[i].offset.x.toInt(), glyphs[i].offset.y.toInt());
         }
+        item->kerning_applied = kerning_feature_selected;
     } else {
         qt_heuristicPosition(item);
     }
 
 #ifdef OT_DEBUG
     if (doLogClusters) {
-        qDebug("log clusters after shaping:");
+        DEBUG("log clusters after shaping:");
         for (int j = 0; j < length; j++)
-            qDebug("    log[%d] = %d", j, item->log_clusters[j]);
+            DEBUG("    log[%d] = %d", j, item->log_clusters[j]);
     }
-    qDebug("final glyphs:");
-    for (int i = 0; i < (int)otl_buffer->in_length; ++i)
-        qDebug("   glyph=%4x char_index=%d mark: %d cmp: %d, clusterStart: %d advance=%d/%d offset=%d/%d",
-               glyphs[i].glyph, otl_buffer->in_string[i].cluster, glyphs[i].attributes.mark,
+    DEBUG("final glyphs:");
+    for (int i = 0; i < (int)hb_buffer->in_length; ++i)
+        DEBUG("   glyph=%4x char_index=%d mark: %d cmp: %d, clusterStart: %d advance=%d/%d offset=%d/%d",
+               glyphs[i].glyph, hb_buffer->in_string[i].cluster, glyphs[i].attributes.mark,
                glyphs[i].attributes.combiningClass, glyphs[i].attributes.clusterStart,
                glyphs[i].advance.x.toInt(), glyphs[i].advance.y.toInt(),
                glyphs[i].offset.x.toInt(), glyphs[i].offset.y.toInt());
-    qDebug("-----------------------------------------");
+    DEBUG("-----------------------------------------");
 #endif
     return true;
 }
