@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -28,6 +28,8 @@
 #include <QtCore/qobject.h>
 #include <QtGui/qtextobject.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 class QTextCursor;
@@ -43,6 +45,7 @@ public:
     QTextTableCell &operator=(const QTextTableCell &o)
     { table = o.table; fragment = o.fragment; return *this; }
 
+    void setFormat(const QTextCharFormat &format);
     QTextCharFormat format() const;
 
     int row() const;
@@ -114,4 +117,6 @@ private:
 inline void QTextTable::setFormat(const QTextTableFormat &aformat)
 { QTextObject::setFormat(aformat); }
 
+QT_END_HEADER
+
 #endif // QTEXTTABLE_H
