@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -49,15 +49,17 @@ class QPushButtonPrivate : public QAbstractButtonPrivate
 {
     Q_DECLARE_PUBLIC(QPushButton)
 public:
-    QPushButtonPrivate():autoDefault(true), defaultButton(false), flat(false){}
+    enum AutoDefaultValue {Off = 0, On = 1, Auto = 2};
+    QPushButtonPrivate():autoDefault(Auto), defaultButton(false), flat(false), menuOpen(false){}
     void init();
-    void popupPressed();
+    void _q_popupPressed();
     QStyleOptionButton getStyleOption() const;
-    QDialog *dialogParent();
+    QDialog *dialogParent() const;
     QPointer<QMenu> menu;
-    uint autoDefault : 1;
+    uint autoDefault : 2;
     uint defaultButton : 1;
     uint flat : 1;
+    uint menuOpen : 1;
 };
 
 /*!
@@ -145,8 +147,14 @@ public:
     Other classes of buttons are option buttons (see QRadioButton) and
     check boxes (see QCheckBox).
 
-    \inlineimage macintosh-pushbutton.png Screenshot in Macintosh style
-    \inlineimage windows-pushbutton.png Screenshot in Windows style
+    \table 100%
+    \row \o \inlineimage macintosh-pushbutton.png Screenshot of a Macintosh style push button
+         \o A push button shown in the \l{Macintosh Style Widget Gallery}{Macintosh widget style}.
+    \row \o \inlineimage windowsxp-pushbutton.png Screenshot of a Windows XP style push button
+         \o A push button shown in the \l{Windows XP Style Widget Gallery}{Windows XP widget style}.
+    \row \o \inlineimage plastique-pushbutton.png Screenshot of a Plastique style push button
+         \o A push button shown in the \l{Plastique Style Widget Gallery}{Plastique widget style}.
+    \endtable
 
     In Qt, the QAbstractButton base class provides most of the modes
     and other API, and QPushButton provides GUI logic.
@@ -194,8 +202,8 @@ public:
     (depending on the GUI style).
 
     The default button behavior is provided only in dialogs. Buttons
-    can always be clicked from the keyboard by pressing Enter (or
-    Return) or the Spacebar when the button has focus.
+    can always be clicked from the keyboard by pressing Spacebar when
+    the button has focus.
 
     This property's default is false.
 */
@@ -258,14 +266,14 @@ QPushButton::~QPushButton()
 {
 }
 
-QDialog *QPushButtonPrivate::dialogParent()
+QDialog *QPushButtonPrivate::dialogParent() const
 {
-    Q_Q(QPushButton);
-    QWidget *p = q;
+    Q_Q(const QPushButton);
+    const QWidget *p = q;
     while (p && !p->isWindow()) {
         p = p->parentWidget();
-        if (QDialog *dialog = qobject_cast<QDialog*>(p))
-            return dialog;
+        if (const QDialog *dialog = qobject_cast<const QDialog *>(p))
+            return const_cast<QDialog *>(dialog);
     }
     return 0;
 }
@@ -273,7 +281,6 @@ QDialog *QPushButtonPrivate::dialogParent()
 void QPushButtonPrivate::init()
 {
     Q_Q(QPushButton);
-    autoDefault = (dialogParent() != 0);
     q->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
 }
 
@@ -289,11 +296,11 @@ QStyleOptionButton QPushButtonPrivate::getStyleOption() const
     if (menu)
         opt.features |= QStyleOptionButton::HasMenu;
 #endif
-    if (autoDefault || defaultButton)
+    if (q->autoDefault() || defaultButton)
         opt.features |= QStyleOptionButton::AutoDefaultButton;
     if (defaultButton)
         opt.features |= QStyleOptionButton::DefaultButton;
-    if (down)
+    if (down || menuOpen)
         opt.state |= QStyle::State_Sunken;
     if (checked)
         opt.state |= QStyle::State_On;
@@ -308,9 +315,10 @@ QStyleOptionButton QPushButtonPrivate::getStyleOption() const
 void QPushButton::setAutoDefault(bool enable)
 {
     Q_D(QPushButton);
-    if (d->autoDefault == enable)
+    uint state = enable ? QPushButtonPrivate::On : QPushButtonPrivate::Off;
+    if (d->autoDefault != QPushButtonPrivate::Auto && d->autoDefault == state)
         return;
-    d->autoDefault = enable;
+    d->autoDefault = state;
     update();
     updateGeometry();
 }
@@ -318,6 +326,8 @@ void QPushButton::setAutoDefault(bool enable)
 bool QPushButton::autoDefault() const
 {
     Q_D(const QPushButton);
+    if(d->autoDefault == QPushButtonPrivate::Auto)
+        return ( d->dialogParent() != 0 );
     return d->autoDefault;
 }
 
@@ -401,7 +411,7 @@ void QPushButton::keyPressEvent(QKeyEvent *e)
     switch (e->key()) {
     case Qt::Key_Enter:
     case Qt::Key_Return:
-        if (d->autoDefault || d->defaultButton) {
+        if (autoDefault() || d->defaultButton) {
             click();
             break;
         }
@@ -417,7 +427,7 @@ void QPushButton::keyPressEvent(QKeyEvent *e)
 void QPushButton::focusInEvent(QFocusEvent *e)
 {
     Q_D(QPushButton);
-    if (e->reason() != Qt::PopupFocusReason && d->autoDefault && !d->defaultButton) {
+    if (e->reason() != Qt::PopupFocusReason && autoDefault() && !d->defaultButton) {
         d->defaultButton = true;
         QDialog *dlg = qobject_cast<QDialog*>(window());
         if (dlg)
@@ -432,7 +442,7 @@ void QPushButton::focusInEvent(QFocusEvent *e)
 void QPushButton::focusOutEvent(QFocusEvent *e)
 {
     Q_D(QPushButton);
-    if (e->reason() != Qt::PopupFocusReason && d->autoDefault && d->defaultButton) {
+    if (e->reason() != Qt::PopupFocusReason && autoDefault() && d->defaultButton) {
         QDialog *dlg = qobject_cast<QDialog*>(window());
         if (dlg)
             dlg->d_func()->setDefault(0);
@@ -460,11 +470,18 @@ void QPushButton::focusOutEvent(QFocusEvent *e)
 void QPushButton::setMenu(QMenu* menu)
 {
     Q_D(QPushButton);
+    if (menu == d->menu)
+        return;
+
     if (menu && !d->menu) {
-        disconnect(this, SIGNAL(pressed()), this, SLOT(popupPressed()));
-        connect(this, SIGNAL(pressed()), this, SLOT(popupPressed()));
+        disconnect(this, SIGNAL(pressed()), this, SLOT(_q_popupPressed()));
+        connect(this, SIGNAL(pressed()), this, SLOT(_q_popupPressed()));
     }
+    if (d->menu)
+        removeAction(d->menu->menuAction());
     d->menu = menu;
+    if (d->menu)
+        addAction(d->menu->menuAction());
     update();
     updateGeometry();
 }
@@ -492,10 +509,10 @@ void QPushButton::showMenu()
     if (!d || !d->menu)
         return;
     setDown(true);
-    d->popupPressed();
+    d->_q_popupPressed();
 }
 
-void QPushButtonPrivate::popupPressed()
+void QPushButtonPrivate::_q_popupPressed()
 {
     Q_Q(QPushButton);
     if (!down || !menu)
@@ -527,13 +544,16 @@ void QPushButtonPrivate::popupPressed()
         else
             x -= menuSize.width();
     }
-    QObject *guard = q;
-    QMetaObject::addGuard(&guard);
-    menu->exec(QPoint(x, y));
-    if (guard)
-        q->setDown(false);
-    QMetaObject::removeGuard(&guard);
+    QPointer<QPushButton> guard(q);
 
+    //Because of a delay in menu effects, we must keep track of the
+    //menu visibility to avoid flicker on button release
+    menuOpen = true;
+    menu->exec(QPoint(x, y));
+    if (guard) {
+        menuOpen = false;
+        q->setDown(false);
+    }
 }
 #endif // QT_NO_MENU
 
