@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -38,6 +38,10 @@
 #include <qdebug.h>
 
 #include <private/qwidget_p.h>
+
+//### a rubberband window type would be a more elegant solution
+#define RUBBERBAND_WINDOW_TYPE Qt::ToolTip
+
 class QRubberBandPrivate : public QWidgetPrivate
 {
     Q_DECLARE_PUBLIC(QRubberBand)
@@ -58,7 +62,7 @@ QStyleOptionRubberBand QRubberBandPrivate::getStyleOption() const
 #ifndef Q_WS_MAC
     opt.opaque = true;
 #else
-    opt.opaque = q->windowFlags() & Qt::ToolTip;
+    opt.opaque = q->windowFlags() & RUBBERBAND_WINDOW_TYPE;
 #endif
     return opt;
 }
@@ -84,23 +88,23 @@ QStyleOptionRubberBand QRubberBandPrivate::getStyleOption() const
     pattern is to do this in conjunction with mouse events. For example:
 
     \code
-        void Widget::mousePressEvent(QMouseEvent *e)
+        void Widget::mousePressEvent(QMouseEvent *event)
         {
-            origin = e->pos(); // origin is a QPoint
+            origin = event->pos();
             if (!rubberBand)
                 rubberBand = new QRubberBand(QRubberBand::Rectangle, this);
             rubberBand->setGeometry(QRect(origin, QSize()));
-            band->show();
+            rubberBand->show();
         }
 
-        void Widget::mouseMoveEvent(QMouseEvent *e)
+        void Widget::mouseMoveEvent(QMouseEvent *event)
         {
-            band->setGeometry(QRect(origin, e->pos()).normalized());
+            rubberBand->setGeometry(QRect(origin, event->pos()).normalized());
         }
 
-        void Widget::mouseReleaseEvent(QMouseEvent *e)
+        void Widget::mouseReleaseEvent(QMouseEvent *event)
         {
-            band->hide();
+            rubberBand->hide();
             // determine selection, for example using QRect::intersects()
             // and QRect::contains().
         }
@@ -131,17 +135,22 @@ QStyleOptionRubberBand QRubberBandPrivate::getStyleOption() const
     semi-transparent filled selection rectangle.
 */
 QRubberBand::QRubberBand(Shape s, QWidget *p)
-    : QWidget(*new QRubberBandPrivate, p, (p && p->windowType() != Qt::Desktop) ? Qt::Widget : Qt::ToolTip)
+    : QWidget(*new QRubberBandPrivate, p, (p && p->windowType() != Qt::Desktop) ? Qt::Widget : RUBBERBAND_WINDOW_TYPE)
 {
     Q_D(QRubberBand);
     d->shape = s;
     setAttribute(Qt::WA_TransparentForMouseEvents);
+#ifndef Q_WS_WIN
     setAttribute(Qt::WA_NoSystemBackground);
+#endif //Q_WS_WIN
     setAttribute(Qt::WA_WState_ExplicitShowHide);
     setVisible(false);
 #ifdef Q_WS_MAC
-    extern WindowPtr qt_mac_window_for(const QWidget *); //qwidget_mac.cpp
-    ChangeWindowAttributes(qt_mac_window_for(this), kWindowNoShadowAttribute, 0);
+    if(isWindow()) {
+        winId();
+        extern WindowPtr qt_mac_window_for(const QWidget *); //qwidget_mac.cpp
+        ChangeWindowAttributes(qt_mac_window_for(this), kWindowNoShadowAttribute, 0);
+    }
 #endif
 }
 
@@ -214,16 +223,18 @@ void QRubberBand::changeEvent(QEvent *e)
     switch (e->type()) {
     case QEvent::ParentChange:
         if (parent()) {
-            setWindowFlags(windowFlags() & ~Qt::ToolTip);
+            setWindowFlags(windowFlags() & ~RUBBERBAND_WINDOW_TYPE);
         } else {
-            setWindowFlags(windowFlags() | Qt::ToolTip);
+            setWindowFlags(windowFlags() | RUBBERBAND_WINDOW_TYPE);
         }
         break;
     default:
         break;
     }
 
-    raise();
+    // calling raise() on LayoutDirectionChange breaks QWidgetPrivate::setLayoutDirection_helper()
+    if (e->type() != QEvent::LayoutDirectionChange)
+        raise();
 }
 
 /*!
@@ -291,19 +302,27 @@ void QRubberBand::moveEvent(QMoveEvent *)
 */
 
 /*!
-    \fn void QRubberBand::setGeometry(int x, int y, int w, int h)
+    \fn void QRubberBand::setGeometry(const QRect &rect)
 
-    Same as QWidget::setGeometry().
-*/
+    Sets the geometry of the rubber band to \a rect, specified in the coordinate system
+    of its parent widget.
 
-/*!
-    Same as QWidget::setGeometry().
+    \sa QWidget::geometry
 */
 void QRubberBand::setGeometry(const QRect &geom)
 {
     QWidget::setGeometry(geom);
 }
 
+/*!
+    \fn void QRubberBand::setGeometry(int x, int y, int width, int height)
+    \overload
+
+    Sets the geometry of the rubberband to the rectangle whose top-left corner lies at
+    the point (\a x, \a y), and with dimensions specified by \a width and \a height.
+    The geometry is specified in the parent widget's coordinate system.
+*/
+
 /*! \reimp */
 bool QRubberBand::event(QEvent *e)
 {
