@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
@@ -26,6 +26,8 @@
 
 #include <QtGui/qabstractspinbox.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Gui)
 
 #ifndef QT_NO_SPINBOX
@@ -45,19 +47,19 @@ class Q_GUI_EXPORT QSpinBox : public QAbstractSpinBox
 
 public:
     explicit QSpinBox(QWidget *parent = 0);
-#ifdef QT_COMPAT
-    QT_COMPAT_CONSTRUCTOR QSpinBox(QWidget *parent, const char *name);
-    QT_COMPAT_CONSTRUCTOR QSpinBox(int min, int max, int step, QWidget *parent,
-                                   const char *name = 0);
+#ifdef QT3_SUPPORT
+    QT3_SUPPORT_CONSTRUCTOR QSpinBox(QWidget *parent, const char *name);
+    QT3_SUPPORT_CONSTRUCTOR QSpinBox(int min, int max, int step, QWidget *parent,
+                                     const char *name = 0);
 #endif
 
     int value() const;
 
     QString prefix() const;
-    void setPrefix(const QString &p);
+    void setPrefix(const QString &prefix);
 
     QString suffix() const;
-    void setSuffix(const QString &s);
+    void setSuffix(const QString &suffix);
 
     QString cleanText() const;
 
@@ -72,19 +74,19 @@ public:
 
     void setRange(int min, int max);
 
-#ifdef QT_COMPAT
-    inline QT_COMPAT void setLineStep(int step) { setSingleStep(step); }
-    inline QT_COMPAT void setMaxValue(int val) { setMaximum(val); }
-    inline QT_COMPAT void setMinValue(int val) { setMinimum(val); }
-    inline QT_COMPAT int maxValue() const { return maximum(); }
-    inline QT_COMPAT int minValue() const { return minimum(); }
+#ifdef QT3_SUPPORT
+    inline QT3_SUPPORT void setLineStep(int step) { setSingleStep(step); }
+    inline QT3_SUPPORT void setMaxValue(int val) { setMaximum(val); }
+    inline QT3_SUPPORT void setMinValue(int val) { setMinimum(val); }
+    inline QT3_SUPPORT int maxValue() const { return maximum(); }
+    inline QT3_SUPPORT int minValue() const { return minimum(); }
 #endif
 
 protected:
-    bool event(QEvent *e);
+    bool event(QEvent *event);
     virtual QValidator::State validate(QString &input, int &pos) const;
     virtual int valueFromText(const QString &text) const;
-    virtual QString textFromValue(int v) const;
+    virtual QString textFromValue(int val) const;
     virtual void fixup(QString &str) const;
 
 
@@ -112,17 +114,17 @@ class Q_GUI_EXPORT QDoubleSpinBox : public QAbstractSpinBox
     Q_PROPERTY(double maximum READ maximum WRITE setMaximum)
     Q_PROPERTY(double minimum READ minimum WRITE setMinimum)
     Q_PROPERTY(double singleStep READ singleStep WRITE setSingleStep)
-    Q_PROPERTY(double value READ value WRITE setValue NOTIFY valueChanged)
+    Q_PROPERTY(double value READ value WRITE setValue NOTIFY valueChanged USER true)
 public:
     explicit QDoubleSpinBox(QWidget *parent = 0);
 
     double value() const;
 
     QString prefix() const;
-    void setPrefix(const QString &p);
+    void setPrefix(const QString &prefix);
 
     QString suffix() const;
-    void setSuffix(const QString &s);
+    void setSuffix(const QString &suffix);
 
     QString cleanText() const;
 
@@ -142,7 +144,7 @@ public:
 
     virtual QValidator::State validate(QString &input, int &pos) const;
     virtual double valueFromText(const QString &text) const;
-    virtual QString textFromValue(double v) const;
+    virtual QString textFromValue(double val) const;
     virtual void fixup(QString &str) const;
 
 public Q_SLOTS:
@@ -159,4 +161,6 @@ private:
 
 #endif // QT_NO_SPINBOX
 
+QT_END_HEADER
+
 #endif // QSPINBOX_H
