@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -33,7 +33,7 @@
 #include "qsjiscodec.h"
 #include "qlist.h"
 
-#ifndef QT_NO_TEXTCODECPLUGIN
+#ifndef QT_NO_TEXTCODEC
 enum {
     Esc = 0x1b
 };
@@ -91,6 +91,10 @@ QByteArray QSjisCodec::convertFromUnicode(const QChar *uc, int len, ConverterSta
             // JIS X 0208
             *cursor++ = (j >> 8);
             *cursor++ = (j & 0xff);
+        } else if ((j = conv->unicodeToSjisibmvdc(ch.row(), ch.cell())) != 0) {
+            // JIS X 0208 IBM VDC
+            *cursor++ = (j >> 8);
+            *cursor++ = (j & 0xff);
         } else if ((j = conv->unicodeToJisx0212(ch.row(), ch.cell())) != 0) {
             // JIS X 0212 (can't be encoded in ShiftJIS !)
             *cursor++ = 0x81;        // white square
@@ -111,7 +115,7 @@ QByteArray QSjisCodec::convertFromUnicode(const QChar *uc, int len, ConverterSta
 
 QString QSjisCodec::convertToUnicode(const char* chars, int len, ConverterState *state) const
 {
-    uchar buf[1];
+    uchar buf[1] = {0};
     int nbuf = 0;
     QChar replacement = QChar::ReplacementCharacter;
     if (state) {
@@ -121,7 +125,7 @@ QString QSjisCodec::convertToUnicode(const char* chars, int len, ConverterState
         buf[0] = state->state_data[0];
     }
     int invalid = 0;
-
+    uint u= 0;
     QString result;
     for (int i=0; i<len; i++) {
         uchar ch = chars[i];
@@ -129,7 +133,7 @@ QString QSjisCodec::convertToUnicode(const char* chars, int len, ConverterState
         case 0:
             if (ch < 0x80 || IsKana(ch)) {
                 // JIS X 0201 Latin or JIS X 0201 Kana
-                uint u = conv->jisx0201ToUnicode(ch);
+                u = conv->jisx0201ToUnicode(ch);
                 result += QValidChar(u);
             } else if (IsSjisChar1(ch)) {
                 // JIS X 0208
@@ -144,10 +148,13 @@ QString QSjisCodec::convertToUnicode(const char* chars, int len, ConverterState
         case 1:
             // JIS X 0208
             if (IsSjisChar2(ch)) {
-                if (IsUserDefinedChar1(buf[0])) {
+                if ((u = conv->sjisibmvdcToUnicode(buf[0], ch))) {
+                    result += QValidChar(u);
+                }
+                else if (IsUserDefinedChar1(buf[0])) {
                     result += QChar::ReplacementCharacter;
                 } else {
-                    uint u = conv->sjisToUnicode(buf[0], ch);
+                    u = conv->sjisToUnicode(buf[0], ch);
                     result += QValidChar(u);
                 }
             } else {
@@ -189,4 +196,4 @@ QList<QByteArray> QSjisCodec::_aliases()
          << "MS_Kanji";
     return list;
 }
-#endif // QT_NO_TEXTCODECPLUGIN
+#endif // QT_NO_TEXTCODEC
