@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -29,6 +29,8 @@
 #include <Qt3Support/q3dict.h>
 #include <Qt3Support/q3valuelist.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Qt3SupportLight)
 
 #ifndef QT_NO_NETWORKPROTOCOL
@@ -48,12 +50,12 @@ public:
     virtual Q3NetworkProtocol *createObject() = 0;
 };
 
-template< class Protocol >
+template< class T >
 class Q3NetworkProtocolFactory : public Q3NetworkProtocolFactoryBase
 {
 public:
     Q3NetworkProtocol *createObject() {
-	return new Protocol;
+	return new T;
     }
 
 };
@@ -221,4 +223,6 @@ private:	// Disabled copy constructor and operator=
 
 #endif // QT_NO_NETWORKPROTOCOL
 
+QT_END_HEADER
+
 #endif // Q3NETWORKPROTOCOL_H
