@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -619,7 +619,7 @@ void Q3Socket::tryConnecting()
     \sa close()
 */
 
-bool Q3Socket::open( int m )
+bool Q3Socket::open( OpenMode m )
 {
     if ( isOpen() ) {
 #if defined(QT_CHECK_STATE)
@@ -627,10 +627,14 @@ bool Q3Socket::open( int m )
 #endif
 	return false;
     }
-    QIODevice::setOpenMode( OpenMode(m & IO_ReadWrite) );
+    QIODevice::setOpenMode( m & ReadWrite );
     return true;
 }
 
+/*!
+    \fn bool Q3Socket::open(int m)
+    \overload
+*/
 
 /*!
     Closes the socket.
@@ -870,7 +874,7 @@ qint64 Q3Socket::bytesAvailable() const
     Q3Socket * that = (Q3Socket *)this;
     if ( that->d->socket->bytesAvailable() ) // a little slow, perhaps...
 	(void)that->sn_read();
-    return that->d->rba.size();
+    return that->d->rba.size() + QIODevice::bytesAvailable();
 }
 
 
@@ -1229,8 +1233,11 @@ void Q3Socket::sn_read( bool force )
 	    d->connectionClosed();
 	    emit connectionClosed();
 	    Q3SocketPrivate::sn_read_alreadyCalled.removeRef( this );
+            delete a;
 	    return;
 	} else if ( nread < 0 ) {
+            delete a;
+
 	    if ( d->socket->error() == Q3SocketDevice::NoError ) {
 		// all is fine
 		Q3SocketPrivate::sn_read_alreadyCalled.removeRef( this );
@@ -1239,7 +1246,6 @@ void Q3Socket::sn_read( bool force )
 #if defined(QT_CHECK_RANGE)
 	    qWarning( "Q3Socket::sn_read: Read error" );
 #endif
-	    delete a;
 	    if ( d->rsn )
 		d->rsn->setEnabled( false );
 	    emit error( ErrSocketRead );
