@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -27,6 +27,8 @@
 #include <QtCore/qiodevice.h>
 #include <QtNetwork/qhostaddress.h> // int->QHostAddress conversion
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Qt3Support)
 
 #ifndef QT_NO_NETWORK
@@ -51,7 +53,8 @@ public:
     int		 socket() const;
     virtual void setSocket( int socket, Type type );
 
-    bool	 open( int mode );
+    bool	 open( OpenMode mode );
+    bool	 open( int mode ) { return open((OpenMode)mode); }
     void	 close();
     bool	 flush();
 
@@ -147,4 +150,6 @@ private:	// Disabled copy constructor and operator=
 
 #endif // QT_NO_NETWORK
 
+QT_END_HEADER
+
 #endif // Q3SOCKETDEVICE_H
