@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -33,6 +33,8 @@
 #include <Qt3Support/q3sqleditorfactory.h>
 #include <Qt3Support/qiconset.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Qt3Support)
 
 #ifndef QT_NO_SQL_VIEW_WIDGETS
@@ -222,4 +224,6 @@ private:
 
 #endif // QT_NO_SQL_VIEW_WIDGETS
 
+QT_END_HEADER
+
 #endif // Q3DATATABLE_H
