@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt3Support module of the Qt Toolkit.
 **
@@ -31,6 +31,8 @@
 #include <QtGui/qcolor.h>
 #include <QtGui/qfont.h>
 
+QT_BEGIN_HEADER
+
 QT_MODULE(Qt3SupportLight)
 
 #ifndef QT_NO_RICHTEXT
@@ -206,4 +208,6 @@ private:
 
 #endif // QT_NO_RICHTEXT
 
+QT_END_HEADER
+
 #endif // Q3STYLESHEET_H
