@@ -160,17 +160,17 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
         }
     }
 
-    QStringList::Iterator it;
+    QStringList::ConstIterator it;
 
     globalIncludes = unique(globalIncludes);
-    for (it = globalIncludes.begin(); it != globalIncludes.end(); ++it) {
+    for (it = globalIncludes.constBegin(); it != globalIncludes.constEnd(); ++it) {
         if (!(*it).isEmpty()) {
             QString header = fixHeaderName(*it);
             out << "#include <" << header << ">" << endl;
         }
     }
     localIncludes = unique(localIncludes);
-    for (it = localIncludes.begin(); it != localIncludes.end(); ++it) {
+    for (it = localIncludes.constBegin(); it != localIncludes.constEnd(); ++it) {
         if (!(*it).isEmpty()) {
             QString header = fixHeaderName(*it);
             out << "#include \"" << header << "\"" << endl;
@@ -184,7 +184,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     if (dbForms[QLatin1String("(default)")].count())
         dbForm = true;
     bool subDbForms = false;
-    for (it = dbConnections.begin(); it != dbConnections.end(); ++it) {
+    for (it = dbConnections.constBegin(); it != dbConnections.constEnd(); ++it) {
         if (!(*it).isEmpty() && (*it) != QLatin1String("(default)")) {
             if (dbForms[(*it)].count()) {
                 subDbForms = true;
@@ -195,7 +195,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
 
     // some typedefs, maybe
     typeDefs = unique(typeDefs);
-    for (it = typeDefs.begin(); it != typeDefs.end(); ++it) {
+    for (it = typeDefs.constBegin(); it != typeDefs.constEnd(); ++it) {
         if (!(*it).isEmpty())
             out << "typedef " << *it << ";" << endl;
     }
@@ -209,15 +209,15 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
         exportMacro = nl.item(0).firstChild().toText().data();
 
     forwardDecl = unique(forwardDecl);
-    for (it = forwardDecl.begin(); it != forwardDecl.end(); ++it) {
+    for (it = forwardDecl.constBegin(); it != forwardDecl.constEnd(); ++it) {
         if (!(*it).isEmpty() && (*it) != objClass) {
             QString forwardName = *it;
             QStringList forwardNamespaces = forwardName.split(QLatin1String("::"));
             forwardName = forwardNamespaces.last();
             forwardNamespaces.removeAt(forwardNamespaces.size()-1);
 
-            QStringList::ConstIterator ns = forwardNamespaces.begin();
-            while (ns != forwardNamespaces.end()) {
+            QStringList::ConstIterator ns = forwardNamespaces.constBegin();
+            while (ns != forwardNamespaces.constEnd()) {
                 out << "namespace " << *ns << " {" << endl;
                 ++ns;
             }
@@ -227,7 +227,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
         }
     }
 
-    for (it = forwardDecl2.begin(); it != forwardDecl2.end(); ++it) {
+    for (it = forwardDecl2.constBegin(); it != forwardDecl2.constEnd(); ++it) {
         QString fd = *it;
         fd = fd.trimmed();
         if (!fd.endsWith(QLatin1String(";")))
@@ -246,8 +246,8 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     d.uic(fileName, ui, &out);
     delete ui;
 
-    QStringList::ConstIterator ns = namespaces.begin();
-    while (ns != namespaces.end()) {
+    QStringList::ConstIterator ns = namespaces.constBegin();
+    while (ns != namespaces.constEnd()) {
         out << "namespace " << *ns << " {" << endl;
         ++ns;
     }
@@ -264,11 +264,11 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
 
     // constructor
     if (objClass == QLatin1String("QDialog") || objClass == QLatin1String("QWizard")) {
-        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);" << endl;
+        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);" << endl;
     } else if (objClass == QLatin1String("QWidget")) {
-        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = 0);" << endl;
+        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = 0);" << endl;
     } else if (objClass == QLatin1String("QMainWindow") || objClass == QLatin1String("Q3MainWindow")) {
-        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WFlags fl = Qt::WType_TopLevel);" << endl;
+        out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0, Qt::WindowFlags fl = Qt::WType_TopLevel);" << endl;
         isMainWindow = true;
     } else {
         out << "    " << bareNameOfClass << "(QWidget* parent = 0, const char* name = 0);" << endl;
@@ -281,7 +281,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     // database connections
     dbConnections = unique(dbConnections);
     bool hadOutput = false;
-    for (it = dbConnections.begin(); it != dbConnections.end(); ++it) {
+    for (it = dbConnections.constBegin(); it != dbConnections.constEnd(); ++it) {
         if (!(*it).isEmpty()) {
             // only need pointers to non-default connections
             if ((*it) != QLatin1String("(default)") && !(*it).isEmpty()) {
@@ -379,7 +379,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     }
 
     if (!publicVars.isEmpty()) {
-        for (it = publicVars.begin(); it != publicVars.end(); ++it)
+        for (it = publicVars.constBegin(); it != publicVars.constEnd(); ++it)
             out << indent << *it << endl;
         out << endl;
     }
@@ -411,14 +411,14 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     // create signals
     if (!extraSignals.isEmpty()) {
         out << "signals:" << endl;
-        for (it = extraSignals.begin(); it != extraSignals.end(); ++it)
+        for (it = extraSignals.constBegin(); it != extraSignals.constEnd(); ++it)
             out << "    void " << (*it) << ";" << endl;
         out << endl;
     }
 
     if (!protectedVars.isEmpty()) {
         out << "protected:" << endl;
-        for (it = protectedVars.begin(); it != protectedVars.end(); ++it)
+        for (it = protectedVars.constBegin(); it != protectedVars.constEnd(); ++it)
             out << indent << *it << endl;
         out << endl;
     }
@@ -443,7 +443,7 @@ void Ui3Reader::createFormDecl(const QDomElement &e)
     if (!privateFuncts.isEmpty() || !privateVars.isEmpty()) {
         out << "private:" << endl;
         if (!privateVars.isEmpty()) {
-            for (it = privateVars.begin(); it != privateVars.end(); ++it)
+            for (it = privateVars.constBegin(); it != privateVars.constEnd(); ++it)
                 out << indent << *it << endl;
             out << endl;
         }
@@ -650,14 +650,14 @@ void Ui3Reader::createFormImpl(const QDomElement &e)
         out << " *  The " << objClass.mid(1).toLower() << " will by default be modeless, unless you set 'modal' to" << endl;
         out << " *  true to construct a modal " << objClass.mid(1).toLower() << "." << endl;
         out << " */" << endl;
-        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, bool modal, Qt::WFlags fl)" << endl;
+        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, bool modal, Qt::WindowFlags fl)" << endl;
         out << "    : " << objClass << "(parent, name, modal, fl)";
     } else if (objClass == QLatin1String("QWidget"))  {
         out << "/*" << endl;
         out << " *  Constructs a " << nameOfClass << " as a child of 'parent', with the" << endl;
         out << " *  name 'name' and widget flags set to 'f'." << endl;
         out << " */" << endl;
-        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WFlags fl)" << endl;
+        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WindowFlags fl)" << endl;
         out << "    : " << objClass << "(parent, name, fl)";
     } else if (objClass == QLatin1String("QMainWindow") || objClass == QLatin1String("Q3MainWindow")) {
         out << "/*" << endl;
@@ -665,7 +665,7 @@ void Ui3Reader::createFormImpl(const QDomElement &e)
         out << " *  name 'name' and widget flags set to 'f'." << endl;
         out << " *" << endl;
         out << " */" << endl;
-        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WFlags fl)" << endl;
+        out << nameOfClass << "::" << bareNameOfClass << "(QWidget* parent, const char* name, Qt::WindowFlags fl)" << endl;
         out << "    : " << objClass << "(parent, name, fl)";
         isMainWindow = true;
     } else {
@@ -717,49 +717,6 @@ void Ui3Reader::createFormImpl(const QDomElement &e)
         }
     }
 
-    for (n = e; !n.isNull(); n = n.nextSibling().toElement()) {
-        if (n.tagName() == QLatin1String("connections")) {
-            // setup signals and slots connections
-            out << endl << indent << "// signals and slots connections" << endl;
-            nl = n.elementsByTagName(QLatin1String("connection"));
-            for (i = 0; i < (int) nl.length(); i++) {
-                QString sender, receiver, signal, slot;
-                for (QDomElement n2 = nl.item(i).firstChild().toElement(); !n2.isNull(); n2 = n2.nextSibling().toElement()) {
-                    if (n2.tagName() == QLatin1String("sender"))
-                        sender = n2.firstChild().toText().data();
-                    else if (n2.tagName() == QLatin1String("receiver"))
-                        receiver = n2.firstChild().toText().data();
-                    else if (n2.tagName() == QLatin1String("signal"))
-                        signal = fixDeclaration(n2.firstChild().toText().data());
-                    else if (n2.tagName() == QLatin1String("slot"))
-                        slot = fixDeclaration(n2.firstChild().toText().data());
-                }
-                if (sender.isEmpty() ||
-                     receiver.isEmpty() ||
-                     signal.isEmpty() ||
-                     slot.isEmpty())
-                    continue;
-                if (sender[0] == QLatin1Char('<') ||
-                     receiver[0] == QLatin1Char('<') ||
-                     signal[0] == QLatin1Char('<') ||
-                     slot[0] == QLatin1Char('<'))
-                    continue;
-
-                sender = registeredName(sender);
-                receiver = registeredName(receiver);
-
-                 // translate formwindow name to "this"
-                if (sender == objName)
-                    sender = QLatin1String("this");
-                if (receiver == objName)
-                    receiver = QLatin1String("this");
-
-                out << indent << "connect(" << sender << ", SIGNAL(" << signal << "), "
-                    << receiver << ", SLOT(" << slot << "));" << endl;
-            }
-        }
-    }
-
     if (extraFuncts.contains(QLatin1String("init()")))
         out << indent << "init();" << endl;
 
