@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
@@ -24,8 +24,6 @@
 #include "mainwindow.h"
 #include "tabbedbrowser.h"
 #include "helpdialog.h"
-#include "finddialog.h"
-#include "settingsdialog.h"
 #include "config.h"
 
 #include <QDockWidget>
@@ -67,11 +65,10 @@ MainWindow::MainWindow()
 
     goActions = QList<QAction*>();
     goActionDocFiles = new QMap<QAction*,QString>;
-
+    
     windows.append(this);
     tabs = new TabbedBrowser(this);
     setCentralWidget(tabs);
-    settingsDia = 0;
 
     Config *config = Config::configuration();
 
@@ -89,26 +86,36 @@ MainWindow::MainWindow()
     // read geometry configuration
     setupGoActions();
 
-    if (!config->isMaximized()) {
-        QRect geom = config->geometry();
-        if(geom.isValid()) {
-            resize(geom.size());
-            move(geom.topLeft());
-        }
-    }
-
+    restoreGeometry(config->windowGeometry());
     restoreState(config->mainWindowState());
     if (config->sideBarHidden())
         dw->hide();
 
     tabs->setup();
     QTimer::singleShot(0, this, SLOT(setup()));
-#if defined(Q_OS_MACX)
+#if defined(Q_WS_MAC)
+    QMenu *windowMenu = new QMenu(tr("&Window"), this);
+    menuBar()->insertMenu(ui.helpMenu->menuAction(), windowMenu);
+    windowMenu->addAction(tr("Minimize"), this,
+        SLOT(showMinimized()), QKeySequence(tr("Ctrl+M")));
     // Use the same forward and backward browser shortcuts as Safari and Internet Explorer do
     // on the Mac. This means that if you have access to one of those cool Intellimice, the thing
     // works just fine, since that's how Microsoft hacked it.
     ui.actionGoPrevious->setShortcut(QKeySequence(Qt::CTRL|Qt::Key_Left));
     ui.actionGoNext->setShortcut(QKeySequence(Qt::CTRL|Qt::Key_Right));
+
+    static const QLatin1String MacIconPath(":/trolltech/assistant/images/mac");
+    ui.actionGoNext->setIcon(QIcon(MacIconPath + QLatin1String("/next.png")));
+    ui.actionGoPrevious->setIcon(QIcon(MacIconPath + QLatin1String("/prev.png")));
+    ui.actionGoHome->setIcon(QIcon(MacIconPath + QLatin1String("/home.png")));
+    ui.actionEditCopy->setIcon(QIcon(MacIconPath + QLatin1String("/editcopy.png")));
+    ui.actionEditCopy->setIcon(QIcon(MacIconPath + QLatin1String("/editcopy.png")));
+    ui.actionEditFind->setIcon(QIcon(MacIconPath + QLatin1String("/find.png")));
+    ui.actionFilePrint->setIcon(QIcon(MacIconPath + QLatin1String("/print.png")));
+    ui.actionZoomOut->setIcon(QIcon(MacIconPath + QLatin1String("/zoomout.png")));
+    ui.actionZoomIn->setIcon(QIcon(MacIconPath + QLatin1String("/zoomin.png")));
+    ui.actionSyncToc->setIcon(QIcon(MacIconPath + QLatin1String("/synctoc.png")));
+    ui.actionHelpWhatsThis->setIcon(QIcon(MacIconPath + QLatin1String("/whatsthis.png")));
 #endif
 }
 
@@ -158,6 +165,9 @@ void MainWindow::setup()
     connect(new QShortcut(tr("Ctrl+I"), this), SIGNAL(activated()), helpDock, SLOT(toggleIndex()));
     connect(new QShortcut(tr("Ctrl+B"), this), SIGNAL(activated()), helpDock, SLOT(toggleBookmarks()));
     connect(new QShortcut(tr("Ctrl+S"), this), SIGNAL(activated()), helpDock, SLOT(toggleSearch()));
+    connect(new QShortcut(tr("Ctrl+W"), this), SIGNAL(activated()), tabs, SLOT(closeTab()));
+    connect(new QShortcut(tr("Ctrl+]"), this), SIGNAL(activated()), tabs, SLOT(nextTab()));
+    connect(new QShortcut(tr("Ctrl+["), this), SIGNAL(activated()), tabs, SLOT(previousTab()));
 
     Config *config = Config::configuration();
 
@@ -167,9 +177,16 @@ void MainWindow::setup()
     viewsAction->setText(tr("Views"));
     ui.viewMenu->addAction(viewsAction);
 
-    helpDock->tabWidget()->setCurrentIndex(config->sideBarPage());
-
-    qApp->restoreOverrideCursor();
+    const int tabIndex = config->sideBarPage();
+    helpDock->tabWidget()->setCurrentIndex(tabIndex);
+    // The tab index is 0 by default, so we need to force an upate
+    // to poulate the contents in this case.
+    if (tabIndex == 0) 
+        helpDock->currentTabChanged(tabIndex);
+    QObject::connect(ui.actionEditFind, SIGNAL(triggered()), tabs, SLOT(find()));
+    QObject::connect(ui.actionEditFindNext, SIGNAL(triggered()), tabs, SLOT(findNext()));
+    QObject::connect(ui.actionEditFindPrev, SIGNAL(triggered()), tabs, SLOT(findPrevious()));
+	qApp->restoreOverrideCursor();
     ui.actionGoPrevious->setEnabled(false);
     ui.actionGoNext->setEnabled(false);
 }
@@ -209,6 +226,7 @@ void MainWindow::setupGoActions()
             }
             action = new QAction(this);
             action->setText(title);
+            action->setWhatsThis(tr("Displays the main page of a specific documentation set."));
             action->setIcon(QIcon(pix));
             ui.goMenu->addAction(action);
             ui.goActionToolbar->addAction(action);
@@ -242,28 +260,34 @@ void MainWindow::closeEvent(QCloseEvent *e)
 void MainWindow::about()
 {
     QMessageBox box(this);
-    box.setText(tr("<center><img src=\":/trolltech/assistant/images/assistant-128.png\">"
-                   "<h3>%1</h3>"
-                   "<br/>Version %2"
-#if defined(QT_OPENSOURCE)
-                   " Open Source Edition</center><p>"
-                   "This version of Qt Assistant is part of the Qt Open Source Edition, for use "
-                   "in the development of Open Source applications. "
-                   "Qt is a comprehensive C++ framework for cross-platform application "
-                   "development.<br/><br/>"
-                   "You need a commercial Qt license for development of proprietary (closed "
-                   "source) applications. Please see <tt>http://www.trolltech.com/company/model"
-                   ".html</tt> for an overview of Qt licensing."
-#else
-                   "</center><p>This program is licensed to you under the terms of the "
+#if QT_EDITION == QT_EDITION_OPENSOURCE
+    QString edition = tr("Open Source Edition");
+    QString info = tr("This version of Qt Assistant is part of the Qt Open Source Edition, for use "
+                      "in the development of Open Source applications. "
+                      "Qt is a comprehensive C++ framework for cross-platform application "
+                      "development.");
+    QString moreInfo = tr("You need a commercial Qt license for development of proprietary (closed "
+                   "source) applications. Please see <a href=\"http://www.trolltech.com/company/model"
+                   ".html\">www.trolltech.com/company/model.html</a> for an overview of Qt licensing.");
+#else 
+    QString edition;
+    QString info;
+    QString moreInfo(tr("This program is licensed to you under the terms of the "
                    "Qt Commercial License Agreement. For details, see the file LICENSE "
-                   "that came with this software distribution."
+                   "that came with this software distribution."));
+
 #endif
-                   "<br/><br/>Copyright 2000-2005 Trolltech AS. All rights reserved."
-                   "<br/><br/>The program is provided AS IS with NO WARRANTY OF ANY KIND,"
+
+    box.setText(QString("<center><img src=\":/trolltech/assistant/images/assistant-128.png\">"
+                   "<h3>%1</h3>"
+                   "<p>Version %2 %3</p></center>"
+                   "<p>%4</p>"
+                   "<p>%5</p>"
+                   "<p>Copyright (C) 2000-2006 Trolltech ASA. All rights reserved.</p>"
+                   "<p>The program is provided AS IS with NO WARRANTY OF ANY KIND,"
                    " INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A"
-                   " PARTICULAR PURPOSE.<br/> ")
-                   .arg(tr("Qt Assistant")).arg(QT_VERSION_STR));
+                   " PARTICULAR PURPOSE.<p/>")
+                   .arg(tr("Qt Assistant")).arg(QT_VERSION_STR).arg(edition).arg(info).arg(moreInfo));
     box.setWindowTitle(tr("Qt Assistant"));
     box.setIcon(QMessageBox::NoIcon);
     box.exec();
@@ -281,7 +305,7 @@ void MainWindow::on_actionAboutApplication_triggered()
         url = url.mid(5);
     QFile file(url);
     if(file.exists() && file.open(QFile::ReadOnly))
-        text = QString::fromAscii(file.readAll());
+        text = QString::fromUtf8(file.readAll());
     if(text.isNull())
         text = tr("Failed to open about application contents in file: '%1'").arg(url);
 
@@ -297,34 +321,6 @@ void MainWindow::on_actionAboutAssistant_triggered()
     about();
 }
 
-void MainWindow::on_actionEditFind_triggered()
-{
-    if (!findDialog)
-        findDialog = new FindDialog(this);
-    findDialog->reset();
-    findDialog->show();
-    findDialog->raise();
-    findDialog->activateWindow();
-}
-
-void MainWindow::on_actionEditFindAgain_triggered()
-{
-    if (!findDialog || !findDialog->hasFindExpression()) {
-        on_actionEditFind_triggered();
-        return;
-    }
-    findDialog->doFind(true);
-}
-
-void MainWindow::on_actionEditFindAgainPrev_triggered()
-{
-    if (!findDialog || !findDialog->hasFindExpression()) {
-        on_actionEditFind_triggered();
-        return;
-    }
-    findDialog->doFind(false);
-}
-
 void MainWindow::on_actionGoHome_triggered()
 {
     QString home = MainWindow::urlifyFileName(Config::configuration()->homePage());
@@ -338,10 +334,10 @@ QString MainWindow::urlifyFileName(const QString &fileName)
 #if defined(Q_OS_WIN32)
     if (!url.isValid() || url.scheme().isEmpty() || url.scheme().toLower() != "file:") {
         name = name.toLower();
+        name.replace('\\', '/');
         foreach (QFileInfo drive, QDir::drives()) {
             if (name.startsWith(drive.absolutePath().toLower())) {
                 name = "file:" + name;
-                name = name.replace("\\", "/");
                 break;
             }
         }
@@ -433,18 +429,22 @@ void MainWindow::showLinks(const QStringList &links)
         return;
     }
 
-    pendingLinks = links;
-
-    QStringList::ConstIterator it = pendingLinks.begin();
+    QStringList::ConstIterator it = links.begin();
     // Initial showing, The tab is empty so update that without creating it first
     if (!tabs->currentBrowser()->source().isValid()) {
-        pendingBrowsers.append(tabs->currentBrowser());
-        tabs->setTitle(tabs->currentBrowser(), pendingLinks.first());
+        QPair<HelpWindow*, QString> browser;
+        browser.first = tabs->currentBrowser();
+        browser.second = links.first();
+        pendingBrowsers.append(browser);
+        tabs->setTitle(tabs->currentBrowser(), tr("..."));
     }
     ++it;
 
-    while(it != pendingLinks.end()) {
-        pendingBrowsers.append(tabs->newBackgroundTab(*it));
+    while(it != links.end()) {
+        QPair<HelpWindow*, QString> browser;
+        browser.first = tabs->newBackgroundTab();
+        browser.second = *it;
+        pendingBrowsers.append(browser);
         ++it;
     }
 
@@ -452,16 +452,30 @@ void MainWindow::showLinks(const QStringList &links)
     return;
 }
 
+void MainWindow::removePendingBrowser(HelpWindow *win)
+{
+    if (!pendingBrowsers.count())
+        return;
+
+    QMutableListIterator<QPair<HelpWindow*, QString> > it(pendingBrowsers);
+    while (it.hasNext()) {
+        QPair<HelpWindow*, QString> browser = it.next();
+        if (browser.first == win) {
+            it.remove();
+            break;
+        }
+    }
+}
+
 void MainWindow::timerEvent(QTimerEvent *e)
 {
-    QString link = pendingLinks.first();
-    HelpWindow *win = pendingBrowsers.first();
-    pendingLinks.pop_front();
-    pendingBrowsers.removeFirst();
-    if (pendingLinks.size() == 0)
+    QPair<HelpWindow*, QString> browser = pendingBrowsers.first();
+    pendingBrowsers.pop_front();
+    
+    if (pendingBrowsers.size() == 0)
         killTimer(e->timerId());
 
-    win->setSource(MainWindow::urlifyFileName(link));
+    browser.first->setSource(MainWindow::urlifyFileName(browser.second));
 }
 
 void MainWindow::showQtHelp()
@@ -470,33 +484,6 @@ void MainWindow::showQtHelp()
              QLatin1String("/html/index.html"));
 }
 
-void MainWindow::on_actionSettings_triggered()
-{
-    showSettingsDialog(-1);
-}
-
-void MainWindow::showPDFReaderSettings()
-{
-    showSettingsDialog(2);
-}
-
-void MainWindow::showWebBrowserSettings()
-{
-    showSettingsDialog(1);
-}
-
-void MainWindow::showSettingsDialog(int page)
-{
-    if (!settingsDia){
-        settingsDia = new SettingsDialog(this);
-    }
-
-    if (page != -1)
-        settingsDia->settingsTab()->setCurrentIndex(page);
-
-    settingsDia->exec();
-}
-
 MainWindow* MainWindow::newWindow()
 {
     saveSettings();
@@ -515,10 +502,9 @@ void MainWindow::saveSettings()
     Config *config = Config::configuration();
 
     config->setSideBarPage(helpDock->tabWidget()->currentIndex());
-    config->setGeometry(QRect(x(), y(), width(), height()));
-    config->setMaximized(isMaximized());
+    config->setWindowGeometry(saveGeometry());
     config->setMainWindowState(saveState());
-
+    
     // Create list of the tab urls
     QStringList lst;
     QList<HelpWindow*> browsers = tabs->browsers();
@@ -653,6 +639,17 @@ void MainWindow::setupPopupMenu(QMenu *m)
     m->addAction(ui.actionEditFind);
 }
 
+void MainWindow::on_actionSyncToc_triggered()
+{
+    HelpWindow *w = tabs->currentBrowser();
+    if(w) {
+        qApp->setOverrideCursor(QCursor(Qt::WaitCursor));
+        QString  link = w->source().toString();
+        helpDock->locateContents(link);
+     	qApp->restoreOverrideCursor();
+    }
+}
+
 void MainWindow::on_actionNewWindow_triggered()
 {
     newWindow()->show();
@@ -732,8 +729,9 @@ void MainWindow::on_actionSaveAs_triggered()
             }
         }
     }
-    QString src = doc->toHtml();
+    QString src = doc->toHtml("utf-8");
     QTextStream s(&file);
+    s.setCodec("utf-8");
     s << src;
     s.flush();
     file.close();
