@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Assistant of the Qt Toolkit.
 **
@@ -30,8 +30,6 @@
 
 class TabbedBrowser;
 class HelpDialog;
-class FindDialog;
-class SettingsDialog;
 class HelpWindow;
 class QMenu;
 class QDockWidget;
@@ -49,6 +47,8 @@ public:
     void setupPopupMenu(QMenu *menu);
     static QString urlifyFileName(const QString &fileName);
 
+    void removePendingBrowser(HelpWindow *win);
+
 public slots:
     MainWindow *newWindow();
 
@@ -61,26 +61,20 @@ public slots:
 private slots:
     void on_actionNewWindow_triggered();
     void on_actionGoHome_triggered();
-    void on_actionEditFind_triggered();
-    void on_actionEditFindAgain_triggered();
-    void on_actionEditFindAgainPrev_triggered();
     void on_actionFilePrint_triggered();
-    void on_actionSettings_triggered();
     void on_actionClose_triggered();
     void on_actionHelpWhatsThis_triggered();
     void on_actionHelpAssistant_triggered();
     void on_actionAboutApplication_triggered();
     void on_actionAboutAssistant_triggered();
     void on_actionSaveAs_triggered();
+    void on_actionSyncToc_triggered();
 
     void about();
     void setupBookmarkMenu();
     void showBookmark(QAction *action);
     void showLinkFromClient(const QString &link);
     void showQtHelp();
-    void showWebBrowserSettings();
-    void showPDFReaderSettings();
-    void showSettingsDialog(int page);
     void showSearchLink(const QString &link, const QStringList &terms);
     void showGoActionLink();
     void updateProfileSettings();
@@ -103,15 +97,12 @@ private:
     QList<QAction*> goActions;
     uint setupCompleted:1;
     TabbedBrowser *tabs;
-    SettingsDialog *settingsDia;
     QMap<QAction*, QString> bookmarks;
     HelpDialog *helpDock;
     QDockWidget *dw;
-    QPointer<FindDialog> findDialog;
     static QList<MainWindow*> windows;
     QMap<QAction*,QString> *goActionDocFiles;
-    QStringList pendingLinks;
-    QList<HelpWindow*> pendingBrowsers;
+    QList<QPair<HelpWindow*,QString> > pendingBrowsers;
 };
 
 #endif // MAINWINDOW_H
