@@ -30,6 +30,7 @@ class MainWindow;
 class HelpWindow;
 class QStyleSheet;
 class QMimeSourceFactory;
+class QTimer;
 
 class TabbedBrowser : public QWidget
 {
@@ -48,6 +49,10 @@ public:
 
     void setTitle(HelpWindow*, const QString &);
 
+protected:
+	void keyPressEvent(QKeyEvent *);
+	bool eventFilter(QObject *o, QEvent *e);
+
 public slots:
     void init();
     void forward();
@@ -68,6 +73,13 @@ public slots:
     void copy();
     void closeTab();
     void sourceChanged();
+	
+	void find();
+	void findNext();
+	void findPrevious();
+
+private slots:
+	void find(QString, bool forward = false, bool backward = false);
 
 private:
     Ui::TabbedBrowser ui;
@@ -77,6 +89,7 @@ private:
     QString fixedFontFam;
     QColor lnkColor;
     bool underlineLnk;
+	QTimer *autoHideTimer;
 };
 
 #endif // TABBEDBROWSER_H
