@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -26,6 +26,8 @@
 
 #include <QtCore/qglobal.h>
 
+QT_BEGIN_HEADER
+
 #define QDESIGNER_SDK_EXTERN Q_DECL_EXPORT
 #define QDESIGNER_SDK_IMPORT Q_DECL_IMPORT
 
@@ -37,4 +39,6 @@
 #  define QDESIGNER_SDK_EXPORT QDESIGNER_SDK_IMPORT
 #endif
 
+QT_END_HEADER
+
 #endif // SDK_GLOBAL_H
