@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -21,6 +21,10 @@
 **
 ****************************************************************************/
 
+/*
+TRANSLATOR qdesigner_internal::ResourceModel
+*/
+
 #include <QtCore/QFile>
 #include <QtCore/QTextStream>
 #include <QtCore/QDir>
@@ -394,7 +398,7 @@ QModelIndex ResourceModel::index(int row, int column,
 {
     QModelIndex result;
 
-    qint64 d = parent.internalId();
+    qint32 d = parent.internalId();
 
     if (!parent.isValid()) {
         if (row < m_resource_file.prefixCount())
@@ -413,7 +417,7 @@ QModelIndex ResourceModel::parent(const QModelIndex &index) const
 {
     QModelIndex result;
 
-    qint64 d = index.internalId();
+    qint32 d = index.internalId();
 
     if (index.isValid() && d != -1)
         result = createIndex(d, 0, -1);
@@ -425,7 +429,7 @@ int ResourceModel::rowCount(const QModelIndex &parent) const
 {
     int result = 0;
 
-    qint64 d = parent.internalId();
+    qint32 d = parent.internalId();
 
     if (!parent.isValid())
         result = m_resource_file.prefixCount();
@@ -444,7 +448,7 @@ bool ResourceModel::hasChildren(const QModelIndex &parent) const
 {
     bool result = false;
 
-    qint64 d = parent.internalId();
+    qint32 d = parent.internalId();
 
     if (!parent.isValid())
         result = m_resource_file.prefixCount() > 0;
@@ -476,7 +480,7 @@ QVariant ResourceModel::data(const QModelIndex &index, int role) const
     if (!index.isValid())
         return QVariant();
 
-    qint64 d = index.internalId();
+    qint32 d = index.internalId();
 
     QVariant result;
 
@@ -519,7 +523,7 @@ void ResourceModel::getItem(const QModelIndex &index, QString &prefix, QString &
     if (!index.isValid())
         return;
 
-    qint64 d = index.internalId();
+    qint32 d = index.internalId();
 
     if (d == -1) {
         prefix = m_resource_file.prefix(index.row());
@@ -604,6 +608,9 @@ QModelIndex ResourceModel::addFiles(const QModelIndex &model_idx, const QStringL
     foreach (QString file, file_list)
         m_resource_file.addFile(prefix_idx, file);
 
+    QFileInfo fi(file_list.last());
+    m_lastResourceDir = fi.absolutePath();
+
     endInsertRows();
     setDirty(true);
 
@@ -675,4 +682,11 @@ bool ResourceModel::save()
     return result;
 }
 
+QString ResourceModel::lastResourceOpenDirectory() const
+{
+    if (m_lastResourceDir.isEmpty())
+        return absolutePath(QString());
+    return m_lastResourceDir;
+}
+
 } // namespace qdesigner_internal
