@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2005 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the Qt Designer of the Qt Toolkit.
 **
@@ -26,15 +26,19 @@
 
 #include <QtCore/qglobal.h>
 
-#define QT_UILIB_EXTERN Q_DECL_EXPORT
-#define QT_UILIB_IMPORT Q_DECL_IMPORT
+QT_BEGIN_HEADER
+
+#define QDESIGNER_UILIB_EXTERN Q_DECL_EXPORT
+#define QDESIGNER_UILIB_IMPORT Q_DECL_IMPORT
 
 #ifdef QT_DESIGNER_STATIC
-#  define QT_UILIB_EXPORT
-#elif defined(QT_UILIB_LIBRARY)
-#  define QT_UILIB_EXPORT QT_UILIB_EXTERN
+#  define QDESIGNER_UILIB_EXPORT
+#elif defined(QDESIGNER_UILIB_LIBRARY)
+#  define QDESIGNER_UILIB_EXPORT QDESIGNER_UILIB_EXTERN
 #else
-#  define QT_UILIB_EXPORT QT_UILIB_IMPORT
+#  define QDESIGNER_UILIB_EXPORT QDESIGNER_UILIB_IMPORT
 #endif
 
+QT_END_HEADER
+
 #endif // UILIB_GLOBAL_H
