@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 1992-2006 Trolltech AS. All rights reserved.
+** Copyright (C) 1992-2006 Trolltech ASA. All rights reserved.
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
@@ -59,10 +59,10 @@ static const char *appearance_text =
 "<p>Any GUI Style plugins in your plugin path will automatically be added "
 "to the list of built-in Qt styles. (See the Library Paths tab for "
 "information on adding new plugin paths.)</p>"
-"<p>When you choose 3-D Effects and Background colors, the Qt Configuration "
-"program will automatically generate a palette for you.  To customize "
-"colors further, press the Tune Palette button to open the advanced "
-"palette editor."
+"<p>When you choose 3-D Effects and Window Background colors, the Qt "
+"Configuration program will automatically generate a palette for you. "
+"To customize colors further, press the Tune Palette button to open "
+"the advanced palette editor."
 "<p>The Preview Window shows what the selected Style and colors look "
 "like.";
 
@@ -120,8 +120,8 @@ static const char *printer_text =
 static QColorGroup::ColorRole centralFromItem( int item )
 {
     switch( item ) {
-    case 0:  return QColorGroup::Background;
-    case 1:  return QColorGroup::Foreground;
+    case 0:  return QColorGroup::Window;
+    case 1:  return QColorGroup::WindowText;
     case 2:  return QColorGroup::Button;
     case 3:  return QColorGroup::Base;
     case 4:  return QColorGroup::Text;
@@ -214,7 +214,7 @@ MainWindow::MainWindow()
     buttonMainColor->setColor(palette().color(QPalette::Active,
                                               QColorGroup::Button));
     buttonMainColor2->setColor(palette().color(QPalette::Active,
-                                               QColorGroup::Background));
+                                               QColorGroup::Window));
     connect(buttonMainColor, SIGNAL(colorChanged(QColor)),
                 this, SLOT(buildPalette()));
     connect(buttonMainColor2, SIGNAL(colorChanged(QColor)),
@@ -496,7 +496,7 @@ void MainWindow::buildPalette()
     cg = editPalette.inactive();
 
     QPalette temp( editPalette.active().color( QColorGroup::Button ),
-                   editPalette.active().color( QColorGroup::Background ) );
+                   editPalette.active().color( QColorGroup::Window ) );
 
     for (i = 0; i<9; i++)
         cg.setColor( centralFromItem(i), temp.inactive().color( centralFromItem(i) ) );
@@ -571,7 +571,7 @@ void MainWindow::buildDisabled()
 {
     QColorGroup cg = editPalette.active();
     cg.setColor( QColorGroup::ButtonText, Qt::darkGray );
-    cg.setColor( QColorGroup::Foreground, Qt::darkGray );
+    cg.setColor( QColorGroup::WindowText, Qt::darkGray );
     cg.setColor( QColorGroup::Text, Qt::darkGray );
     cg.setColor( QColorGroup::HighlightedText, Qt::darkGray );
     editPalette.setDisabled( cg );
@@ -632,7 +632,7 @@ void MainWindow::setPreviewPalette( const QPalette& pal )
 void MainWindow::updateColorButtons()
 {
     buttonMainColor->setColor( editPalette.active().color( QColorGroup::Button ));
-    buttonMainColor2->setColor( editPalette.active().color( QColorGroup::Background ));
+    buttonMainColor2->setColor( editPalette.active().color( QColorGroup::Window ));
 }
 
 
@@ -892,7 +892,7 @@ void MainWindow::helpAbout()
                    "Qt Commercial License Agreement. For details, see the file LICENSE "
                    "that came with this software distribution."
 #endif
-                   "<br/><br/>Copyright (C) 2000-2006 Trolltech AS. All rights reserved."
+                   "<br/><br/>Copyright (C) 2000-2006 Trolltech ASA. All rights reserved."
                    "<br/><br/>The program is provided AS IS with NO WARRANTY OF ANY KIND,"
                    " INCLUDING THE WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A"
                    " PARTICULAR PURPOSE.<br/> ")
