@@ -34,7 +34,7 @@ class PaletteEditorAdvancedBase : public QDialog, public Ui::PaletteEditorAdvanc
     Q_OBJECT
 
 public:
-    PaletteEditorAdvancedBase(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WFlags fl = 0);
+    PaletteEditorAdvancedBase(QWidget* parent = 0, const char* name = 0, bool modal = false, Qt::WindowFlags fl = 0);
     ~PaletteEditorAdvancedBase();
 
 protected slots:
