@@ -21,13 +21,7 @@
 **
 ****************************************************************************/
 
-#if !defined( Q_WS_QWS ) || defined( QT_NO_QWS_MULTIPROCESS )
-#define QLock QWSSemaphore
-#undef QT_NO_QWS_MULTIPROCESS
-#include "../../src/gui/embedded/qlock.cpp"
-#else
 #include "qlock_p.h"
-#endif
 
 #include "qvfbshmem.h"
 #include "qvfbhdr.h"
@@ -119,6 +113,7 @@ QShMemViewProtocol::QShMemViewProtocol(int displayid, const QSize &s,
 	case 4:
 	case 8:
 	case 16:
+	case 18:
 	case 24:
 	case 32:
 	    break;
@@ -137,17 +132,19 @@ QShMemViewProtocol::QShMemViewProtocol(int displayid, const QSize &s,
 
     QString oldPipe = "/tmp/qtembedded-" + username + "/" + QString( QTE_PIPE ).arg( displayid );
     int oldPipeSemkey = ftok( oldPipe.toLatin1().constData(), 'd' );
-    int oldPipeLockId = semget( oldPipeSemkey, 0, 0 );
-    if (oldPipeLockId >= 0){
-        sembuf sops;
-        sops.sem_num = 0;
-        sops.sem_op = 1;
-        sops.sem_flg = SEM_UNDO;
-        int rv;
-        do {
-            rv = semop(lockId,&sops,1);
-        } while ( rv == -1 && errno == EINTR );
-        qFatal("Cannot create lock file as an old version of QVFb has opened %s. Close other QVFb and try again", oldPipe.toLatin1().constData());
+    if (oldPipeSemkey != -1) {
+        int oldPipeLockId = semget( oldPipeSemkey, 0, 0 );
+        if (oldPipeLockId >= 0){
+            sembuf sops;
+            sops.sem_num = 0;
+            sops.sem_op = 1;
+            sops.sem_flg = SEM_UNDO;
+            int rv;
+            do {
+                rv = semop(lockId,&sops,1);
+            } while ( rv == -1 && errno == EINTR );
+            qFatal("Cannot create lock file as an old version of QVFb has opened %s. Close other QVFb and try again", oldPipe.toLatin1().constData());
+        }
     }
 
     kh = new QVFbKeyPipeProtocol(displayid);
@@ -162,6 +159,8 @@ QShMemViewProtocol::QShMemViewProtocol(int displayid, const QSize &s,
     int bpl;
     if ( d == 1 )
 	bpl = (w*d+7)/8;
+    else if ( d == 18 )
+        bpl = ((w*24+31)/32)*4;
     else
 	bpl = ((w*actualdepth+31)/32)*4;
 
